@@ -196,7 +196,7 @@ def install_bundle(dest: str = '', which: str = '') -> None:
 
 
 def install_grype() -> str:
-    dest = os.path.join(SW, 'bin')
+    dest = '/tmp'
     rq = Request('https://api.github.com/repos/anchore/grype/releases/latest', headers={
         'Accept': 'application/vnd.github.v3+json',
     })
@@ -211,7 +211,9 @@ def install_grype() -> str:
     data = download_with_retry(url)
     with tarfile.open(fileobj=io.BytesIO(data), mode='r') as tf:
         tf.extract('grype', path=dest, filter='fully_trusted')
-    return os.path.join(dest, 'grype')
+    exe = os.path.join(dest, 'grype')
+    subprocess.check_call([exe, 'db', 'update'])
+    return exe
 
 
 IGNORED_DEPENDENCY_CVES = [
@@ -220,6 +222,8 @@ IGNORED_DEPENDENCY_CVES = [
     'CVE-2025-6069', # DoS in HTMLParser
     # glib
     'CVE-2025-4056', # Only affects Windows, on which we dont run
+    # github.com/nwaples/rardecode/v2
+    'CVE-2025-11579', # rardecode not present in kitty go.sum
 ]
 
 
@@ -236,8 +240,8 @@ def check_dependencies() -> None:
     os.makedirs(dest, exist_ok=True)
     install_bundle(dest, os.path.basename(dest))
     cmdline = [grype, '--by-cve', '--config', gc, '--fail-on', 'medium', '--only-fixed', '--add-cpes-if-none']
-    if (cp := subprocess.run(cmdline + ['dir:' + SW])).returncode != 0:
-        raise SystemExit(cp.returncode)
+    if (subprocess.run(cmdline + ['dir:' + SW])).returncode != 0:
+        raise SystemExit('grype found problems during filesystem scan')
     # Now test against the SBOM
     import runpy
     orig = sys.argv, sys.stdout
@@ -247,8 +251,8 @@ def check_dependencies() -> None:
     runpy.run_path('bypy-src')
     sys.argv, sys.stdout = orig
     print(buf.getvalue())
-    if (cp := subprocess.run(cmdline, input=buf.getvalue().encode())).returncode != 0:
-        raise SystemExit(cp.returncode)
+    if (subprocess.run(cmdline, input=buf.getvalue().encode())).returncode != 0:
+        raise SystemExit('grype found problems during SBOM scan')
 
 
 def main() -> None:
