@@ -46,12 +46,10 @@ detect_os() {
         'Linux')
             OS="linux"
             case "$(command uname -m)" in
-                x86_64) arch="x86_64";;
+                amd64|x86_64) arch="x86_64";;
                 aarch64*) arch="arm64";;
                 armv8*) arch="arm64";;
-                i386) arch="i686";;
-                i686) arch="i686";;
-                *) die "Unknown CPU architecture $(command uname -m)";;
+                *) die "kitty binaries not available for architecture $(command uname -m)";;
             esac
             ;;
         *) die "kitty binaries are not available for $(command uname)"
@@ -100,6 +98,9 @@ get_release_url() {
     get_file_url "v$release_version" "$release_version"
 }
 
+get_version_url() {
+    get_file_url "v$1" "$1"
+}
 
 get_nightly_url() {
     get_file_url "nightly" "nightly"
@@ -110,42 +111,49 @@ get_download_url() {
     case "$installer" in
         "nightly") get_nightly_url ;;
         "") get_release_url ;;
+        version-*) get_version_url "${installer#*-}";;
         *) installer_is_file="y" ;;
     esac
 }
 
-linux_install() {
-    if [ "$installer_is_file" = "y" ]; then
-        command tar -C "$dest" "-xJof" "$installer"
-    else
+download_installer() {
+    tdir=$(command mktemp -d "/tmp/kitty-install-XXXXXXXXXXXX")
+    [ "$installer_is_file" != "y" ] && {
         printf '%s\n\n' "Downloading from: $url"
-        fetch "$url" | command tar -C "$dest" "-xJof" "-"
-    fi
+        if [ "$OS" = "macos" ]; then
+            installer="$tdir/kitty.dmg"
+        else
+            installer="$tdir/kitty.txz"
+        fi
+        fetch "$url" > "$installer" || die "Failed to download: $url"
+        installer_is_file="y"
+    }
+}
+
+ensure_dest() {
+    printf "%s\n" "Installing to $dest"
+    command rm -rf "$dest" || die "Failed to delete $dest"
+    command mkdir -p "$dest" || die "Failed to mkdir -p $dest"
+    command rm -rf "$dest" || die "Failed to delete $dest"
+}
+
+linux_install() {
+    command mkdir "$tdir/mp"
+    command tar -C "$tdir/mp" "-xJof" "$installer" || die "Failed to extract kitty tarball"
+    ensure_dest
+    command mv "$tdir/mp" "$dest" || die "Failed to move kitty.app to $dest"
 }
 
 macos_install() {
-    tdir=$(command mktemp -d "/tmp/kitty-install-XXXXXXXXXXXX")
-    [ "$installer_is_file" != "y" ] && {
-        installer="$tdir/kitty.dmg"
-        printf '%s\n\n' "Downloading from: $url"
-        fetch "$url" > "$installer" || die "Failed to download: $url"
-    }
     command mkdir "$tdir/mp"
     command hdiutil attach "$installer" "-mountpoint" "$tdir/mp" || die "Failed to mount kitty.dmg"
+    ensure_dest
     command ditto -v "$tdir/mp/kitty.app" "$dest"
     rc="$?"
     command hdiutil detach "$tdir/mp"
-    command rm -rf "$tdir"
-    tdir=''
     [ "$rc" != "0" ] && die "Failed to copy kitty.app from mounted dmg"
 }
 
-prepare_install_dest() {
-    printf "%s\n" "Installing to $dest"
-    command rm -rf "$dest"
-    command mkdir -p "$dest" || die "Failed to create the directory: $dest"
-}
-
 exec_kitty() {
     if [ "$OS" = "macos" ]; then
         exec "open" "$dest"
@@ -160,12 +168,13 @@ main() {
     parse_args "$@"
     detect_network_tool
     get_download_url
-    prepare_install_dest
+    download_installer
     if [ "$OS" = "macos" ]; then
         macos_install
     else
         linux_install
     fi
+    cleanup
     [ "$launch" = "y" ] && exec_kitty
     exit 0
 }
