@@ -500,7 +500,6 @@ GLFWAPI GLFWColorScheme glfwGetCurrentSystemColorTheme(bool query_if_unintialize
 
 @interface GLFWApplication : NSApplication
 - (void)tick_callback;
-- (void)render_frame_received:(id)displayIDAsID;
 @end
 
 @implementation GLFWApplication
@@ -509,12 +508,6 @@ GLFWAPI GLFWColorScheme glfwGetCurrentSystemColorTheme(bool query_if_unintialize
     _glfwDispatchTickCallback();
 }
 
-- (void)render_frame_received:(id)displayIDAsID
-{
-    CGDirectDisplayID displayID = [(NSNumber*)displayIDAsID unsignedIntValue];
-    _glfwDispatchRenderFrame(displayID);
-}
-
 @end
 
 
