@@ -64,6 +64,10 @@ class Env:
     vcs_rev: str = ''
     binary_arch: BinaryArch = BinaryArch()
     native_optimizations: bool = False
+    primary_version: int = 0
+    secondary_version: int = 0
+    xt_version: str = ''
+    has_copy_file_range: bool = False
 
     # glfw stuff
     all_headers: List[str] = []
@@ -118,6 +122,10 @@ class Env:
         ans.vcs_rev = self.vcs_rev
         ans.binary_arch = self.binary_arch
         ans.native_optimizations = self.native_optimizations
+        ans.primary_version = self.primary_version
+        ans.secondary_version = self.secondary_version
+        ans.xt_version = self.xt_version
+        ans.has_copy_file_range = self.has_copy_file_range
         return ans
 
 
@@ -315,14 +323,12 @@ def generate_wrappers(glfw_header: str) -> None:
     void glfwWaylandRunWithActivationToken(GLFWwindow *handle, GLFWactivationcallback cb, void *cb_data)
     bool glfwWaylandSetTitlebarColor(GLFWwindow *handle, uint32_t color, bool use_system_color)
     void glfwWaylandRedrawCSDWindowTitle(GLFWwindow *handle)
-    void glfwWaylandSetupLayerShellForNextWindow(GLFWLayerShellConfig c)
+    bool glfwWaylandIsWindowFullyCreated(GLFWwindow *handle)
+    bool glfwWaylandBeep(GLFWwindow *handle)
     pid_t glfwWaylandCompositorPID(void)
-    unsigned long long glfwDBusUserNotify(const char *app_name, const char* icon, const char *summary, const char *body, \
-const char *action_text, int32_t timeout, GLFWDBusnotificationcreatedfun callback, void *data)
+    unsigned long long glfwDBusUserNotify(const GLFWDBUSNotificationData *n, GLFWDBusnotificationcreatedfun callback, void *data)
     void glfwDBusSetUserNotificationHandler(GLFWDBusnotificationactivatedfun handler)
     int glfwSetX11LaunchCommand(GLFWwindow *handle, char **argv, int argc)
-    void glfwSetX11WindowAsDock(int32_t x11_window_id)
-    void glfwSetX11WindowStrut(int32_t x11_window_id, uint32_t dimensions[12])
 '''.splitlines():
         if line:
             functions.append(Function(line.strip(), check_fail=False))
@@ -351,12 +357,12 @@ const char *action_text, int32_t timeout, GLFWDBusnotificationcreatedfun callbac
 typedef int (* GLFWcocoatextinputfilterfun)(int,int,unsigned int,unsigned long);
 typedef bool (* GLFWapplicationshouldhandlereopenfun)(int);
 typedef bool (* GLFWhandleurlopen)(const char*);
-typedef void (* GLFWapplicationwillfinishlaunchingfun)(void);
+typedef void (* GLFWapplicationwillfinishlaunchingfun)(bool);
 typedef bool (* GLFWcocoatogglefullscreenfun)(GLFWwindow*);
 typedef void (* GLFWcocoarenderframefun)(GLFWwindow*);
 typedef void (*GLFWwaylandframecallbackfunc)(unsigned long long id);
 typedef void (*GLFWDBusnotificationcreatedfun)(unsigned long long, uint32_t, void*);
-typedef void (*GLFWDBusnotificationactivatedfun)(uint32_t, const char*);
+typedef void (*GLFWDBusnotificationactivatedfun)(uint32_t, int, const char*);
 {}
 
 const char* load_glfw(const char* path);
