@@ -46,7 +46,7 @@
 #include "internal.h"
 #include "ibus_glfw.h"
 
-#define debug(...) if (_glfw.hints.init.debugKeyboard) printf(__VA_ARGS__);
+#define debug debug_input
 static const char IBUS_SERVICE[]         = "org.freedesktop.IBus";
 static const char IBUS_PATH[]            = "/org/freedesktop/IBus";
 static const char IBUS_INTERFACE[]       = "org.freedesktop.IBus";
@@ -283,29 +283,35 @@ static const char*
 get_ibus_address_file_name(void) {
     const char *addr;
     static char ans[PATH_MAX];
+    static char display[64] = {0};
     addr = getenv("IBUS_ADDRESS");
     int offset = 0;
     if (addr && addr[0]) {
         memcpy(ans, addr, GLFW_MIN(strlen(addr), sizeof(ans)));
         return ans;
     }
-
-    const char *de = getenv("DISPLAY");
-    if (!de || !de[0]) de = ":0.0";
-    char *display = _glfw_strdup(de);
-    const char *host = display;
-    char *disp_num  = strrchr(display, ':');
-    char *screen_num = strrchr(display, '.');
-
-    if (!disp_num) {
-        _glfwInputError(GLFW_PLATFORM_ERROR, "Could not get IBUS address file name as DISPLAY env var has no colon");
-        free(display);
-        return NULL;
+    const char* disp_num = NULL;
+    const char *host = "unix";
+    // See https://github.com/ibus/ibus/commit/8ce25208c3f4adfd290a032c6aa739d2b7580eb1 for why we need this dance.
+    const char *de = getenv("WAYLAND_DISPLAY");
+    if (de) {
+        disp_num = de;
+    } else {
+        const char *de = getenv("DISPLAY");
+        if (!de || !de[0]) de = ":0.0";
+        strncpy(display, de, sizeof(display) - 1);
+        char *dnum = strrchr(display, ':');
+        if (!dnum) {
+            _glfwInputError(GLFW_PLATFORM_ERROR, "Could not get IBUS address file name as DISPLAY env var has no colon");
+            return NULL;
+        }
+        char *screen_num = strrchr(display, '.');
+        *dnum = 0;
+        dnum++;
+        if (screen_num) *screen_num = 0;
+        if (*display) host = display;
+        disp_num = dnum;
     }
-    *disp_num = 0;
-    disp_num++;
-    if (screen_num) *screen_num = 0;
-    if (!*host) host = "unix";
 
     memset(ans, 0, sizeof(ans));
     const char *conf_env = getenv("XDG_CONFIG_HOME");
@@ -315,15 +321,18 @@ get_ibus_address_file_name(void) {
         conf_env = getenv("HOME");
         if (!conf_env || !conf_env[0]) {
             _glfwInputError(GLFW_PLATFORM_ERROR, "Could not get IBUS address file name as no HOME env var is set");
-            free(display);
             return NULL;
         }
         offset = snprintf(ans, sizeof(ans), "%s/.config", conf_env);
     }
-    char *key = dbus_get_local_machine_id();
+    DBusError err;
+    char *key = dbus_try_get_local_machine_id(&err);
+    if (!key) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "Cannot connect to IBUS as could not get DBUS local machine id with error %s: %s", err.name ? err.name : "", err.message ? err.message : "");
+        return NULL;
+    }
     snprintf(ans + offset, sizeof(ans) - offset, "/ibus/bus/%s-%s-%s", key, host, disp_num);
     dbus_free(key);
-    free(display);
     return ans;
 }
 
@@ -364,9 +373,9 @@ read_ibus_address(_GLFWIBUSData *ibus) {
 }
 
 void
-input_context_created(DBusMessage *msg, const char* errmsg, void *data) {
-    if (errmsg) {
-        _glfwInputError(GLFW_PLATFORM_ERROR, "IBUS: Failed to create input context with error: %s", errmsg);
+input_context_created(DBusMessage *msg, const DBusError *err, void *data) {
+    if (err) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "IBUS: Failed to create input context with error: %s: %s", err->name, err->message);
         return;
     }
     const char *path = NULL;
@@ -479,15 +488,15 @@ glfw_ibus_set_cursor_geometry(_GLFWIBUSData *ibus, int x, int y, int w, int h) {
 }
 
 void
-key_event_processed(DBusMessage *msg, const char* errmsg, void *data) {
+key_event_processed(DBusMessage *msg, const DBusError *err, void *data) {
     uint32_t handled = 0;
     _GLFWIBUSKeyEvent *ev = (_GLFWIBUSKeyEvent*)data;
     // Restore key's text from the text embedded in the structure.
     ev->glfw_ev.text = ev->__embedded_text;
     bool is_release = ev->glfw_ev.action == GLFW_RELEASE;
     bool failed = false;
-    if (errmsg) {
-        _glfwInputError(GLFW_PLATFORM_ERROR, "IBUS: Failed to process key with error: %s", errmsg);
+    if (err) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "IBUS: Failed to process key with error: %s: %s", err->name, err->message);
         failed = true;
     } else {
         glfw_dbus_get_args(msg, "Failed to get IBUS handled key from reply", DBUS_TYPE_BOOLEAN, &handled, DBUS_TYPE_INVALID);
