@@ -15,26 +15,82 @@
 #define DESKTOP_INTERFACE "org.freedesktop.portal.Settings"
 #define GNOME_DESKTOP_NAMESPACE "org.gnome.desktop.interface"
 #define FDO_DESKTOP_NAMESPACE "org.freedesktop.appearance"
+static const char* supported_namespaces[2] = {FDO_DESKTOP_NAMESPACE, GNOME_DESKTOP_NAMESPACE};
 #define FDO_APPEARANCE_KEY "color-scheme"
 
 
 static char theme_name[128] = {0};
 static int theme_size = -1;
 static GLFWColorScheme appearance = GLFW_COLOR_SCHEME_NO_PREFERENCE;
-static bool cursor_theme_changed = false;
+static bool cursor_theme_changed = false, appearance_initialized = false;
 
-GLFWColorScheme
-glfw_current_system_color_theme(void) {
-    return appearance;
-}
-
-#define HANDLER(name) static void name(DBusMessage *msg, const char* errmsg, void *data) { \
+#define HANDLER(name_) static void name_(DBusMessage *msg, const DBusError* err, void *data) { \
     (void)data; \
-    if (errmsg) { \
-        _glfwInputError(GLFW_PLATFORM_ERROR, "%s: failed with error: %s", #name, errmsg); \
+    if (err) { \
+        _glfwInputError(GLFW_PLATFORM_ERROR, "%s: failed with error: %s: %s", #name_, err->name, err->message); \
         return; \
     }
 
+HANDLER(get_color_scheme_legacy)
+    DBusMessageIter iter, variant_iter, variant_iter2;
+    if (!dbus_message_iter_init(msg, &iter)) return;
+    dbus_message_iter_recurse(&iter, &variant_iter);
+    int type = dbus_message_iter_get_arg_type(&variant_iter);
+    if (type != DBUS_TYPE_VARIANT) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "Read for color-scheme did not return a variant"); return;
+    }
+    dbus_message_iter_recurse(&variant_iter, &variant_iter2);
+    if (type != DBUS_TYPE_VARIANT) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "Read for color-scheme did not return a nested variant"); return;
+    }
+    uint32_t val;
+    dbus_message_iter_get_basic(&variant_iter2, &val);
+    if (val < 3) appearance = val;
+}
+
+static void
+get_color_scheme(DBusMessage *msg, const DBusError* err, void *data) {
+    (void) data;
+    if (err) {
+        if (strcmp("org.freedesktop.DBus.Error.UnknownMethod", err->name) == 0) {
+            DBusConnection *session_bus = glfw_dbus_session_bus();
+            if (session_bus) {
+                const char *namespace = FDO_DESKTOP_NAMESPACE, *key = FDO_APPEARANCE_KEY;
+                glfw_dbus_call_blocking_method(session_bus, DESKTOP_SERVICE, DESKTOP_PATH, DESKTOP_INTERFACE, "Read", DBUS_TIMEOUT_USE_DEFAULT,
+                    get_color_scheme_legacy, NULL, DBUS_TYPE_STRING, &namespace, DBUS_TYPE_STRING, &key, DBUS_TYPE_INVALID);
+            }
+            return;
+        } else {
+            _glfwInputError(GLFW_PLATFORM_ERROR, "%s: failed with error: %s: %s", "get_color_scheme", err->name, err->message);
+            return;
+        }
+    }
+    uint32_t val;
+    DBusMessageIter iter, variant_iter;
+    if (!dbus_message_iter_init(msg, &iter)) return;
+    dbus_message_iter_recurse(&iter, &variant_iter);
+    int type = dbus_message_iter_get_arg_type(&variant_iter);
+    if (type != DBUS_TYPE_UINT32) {
+        _glfwInputError(GLFW_PLATFORM_ERROR, "ReadOne for color-scheme did not return a uint32"); return;
+    }
+    dbus_message_iter_get_basic(&variant_iter, &val);
+    if (val < 3) appearance = val;
+}
+
+GLFWColorScheme
+glfw_current_system_color_theme(bool query_if_unintialized) {
+    if (!appearance_initialized && query_if_unintialized) {
+        appearance_initialized = true;
+        DBusConnection *session_bus = glfw_dbus_session_bus();
+        if (session_bus) {
+            const char *namespace = FDO_DESKTOP_NAMESPACE, *key = FDO_APPEARANCE_KEY;
+            glfw_dbus_call_blocking_method(session_bus, DESKTOP_SERVICE, DESKTOP_PATH, DESKTOP_INTERFACE, "ReadOne", DBUS_TIMEOUT_USE_DEFAULT,
+                get_color_scheme, NULL, DBUS_TYPE_STRING, &namespace, DBUS_TYPE_STRING, &key, DBUS_TYPE_INVALID);
+        }
+    }
+    return appearance;
+}
+
 static void
 process_fdo_setting(const char *key, DBusMessageIter *value) {
     if (strcmp(key, FDO_APPEARANCE_KEY) == 0) {
@@ -42,7 +98,13 @@ process_fdo_setting(const char *key, DBusMessageIter *value) {
             uint32_t val;
             dbus_message_iter_get_basic(value, &val);
             if (val > 2) val = 0;
-            appearance = val;
+            if (!appearance_initialized) {
+                appearance_initialized = true;
+                if (val != appearance) {
+                    appearance = val;
+                    _glfwInputColorScheme(appearance, true);
+                }
+            }
         }
     }
 }
@@ -111,22 +173,25 @@ HANDLER(process_desktop_settings)
         if (!dbus_message_iter_next(&array)) break;
     }
 #undef die
+#ifndef _GLFW_X11
     if (cursor_theme_changed) _glfwPlatformChangeCursorTheme();
+#endif
 }
 
 #undef HANDLER
 
 static bool
 read_desktop_settings(DBusConnection *session_bus) {
-    DBusMessage *msg = dbus_message_new_method_call(DESKTOP_SERVICE, DESKTOP_PATH, DESKTOP_INTERFACE, "ReadAll");
+    RAII_MSG(msg, dbus_message_new_method_call(DESKTOP_SERVICE, DESKTOP_PATH, DESKTOP_INTERFACE, "ReadAll"));
     if (!msg) return false;
     DBusMessageIter iter, array_iter;
     dbus_message_iter_init_append(msg, &iter);
-    if (!dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "s", &array_iter)) { dbus_message_unref(msg); return false; }
-    if (!dbus_message_iter_close_container(&iter, &array_iter)) { dbus_message_unref(msg); return false; }
-    bool ok = call_method_with_msg(session_bus, msg, DBUS_TIMEOUT_USE_DEFAULT, process_desktop_settings, NULL);
-    dbus_message_unref(msg);
-    return ok;
+    if (!dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "s", &array_iter)) { return false; }
+    for (unsigned i = 0; i < arraysz(supported_namespaces); ++i) {
+        if (!dbus_message_iter_append_basic(&array_iter, DBUS_TYPE_STRING, &supported_namespaces[i])) return false;
+    }
+    if (!dbus_message_iter_close_container(&iter, &array_iter)) { return false; }
+    return call_method_with_msg(session_bus, msg, DBUS_TIMEOUT_USE_DEFAULT, process_desktop_settings, NULL, false);
 }
 
 void
@@ -161,7 +226,8 @@ on_color_scheme_change(DBusMessage *message) {
                 if (val > 2) val = 0;
                 if (val != appearance) {
                     appearance = val;
-                    _glfwInputColorScheme(appearance);
+                    appearance_initialized = true;
+                    _glfwInputColorScheme(appearance, false);
                 }
             }
             break;
