@@ -15,8 +15,8 @@ import (
 	"sync"
 	"unicode"
 
-	"kitty/tools/cli"
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/utils"
 
 	"golang.org/x/sys/unix"
 )
@@ -43,16 +43,18 @@ func get_options_for_rg() (expecting_args map[string]bool, alias_map map[string]
 		if options_started {
 			s := strings.TrimLeft(line, " ")
 			indent := len(line) - len(s)
-			if indent < 12 && indent > 0 {
-				s, _, expecting_arg := strings.Cut(s, "<")
+			if indent < 8 && indent > 0 {
+				expecting_arg := strings.Contains(s, "=")
 				single_letter_aliases := make([]string, 0, 1)
 				long_option_names := make([]string, 0, 1)
 				for _, x := range strings.Split(s, ",") {
 					x = strings.TrimSpace(x)
 					if strings.HasPrefix(x, "--") {
-						long_option_names = append(long_option_names, x[2:])
+						lon, _, _ := strings.Cut(x[2:], "=")
+						long_option_names = append(long_option_names, lon)
 					} else if strings.HasPrefix(x, "-") {
-						single_letter_aliases = append(single_letter_aliases, x[1:])
+						son, _, _ := strings.Cut(x[1:], " ")
+						single_letter_aliases = append(single_letter_aliases, son)
 					}
 				}
 				if len(long_option_names) == 0 {
@@ -68,11 +70,15 @@ func get_options_for_rg() (expecting_args map[string]bool, alias_map map[string]
 				expecting_args[long_option_names[0]] = expecting_arg
 			}
 		} else {
-			if strings.HasPrefix(line, "OPTIONS:") {
+			if strings.HasSuffix(line, "OPTIONS:") {
 				options_started = true
 			}
 		}
 	}
+	if len(expecting_args) == 0 || len(alias_map) == 0 {
+		err = fmt.Errorf("Failed to parse rg help output, could not find any options")
+		return
+	}
 	return
 }
 
@@ -421,6 +427,28 @@ func specialize_command(hg *cli.Command) {
 	hg.ArgCompleter = cli.CompletionForWrapper("rg")
 }
 
+type Options struct {
+}
+
+func create_cmd(root *cli.Command, run_func func(*cli.Command, *Options, []string) (int, error)) {
+	ans := root.AddSubCommand(&cli.Command{
+		Name: "hyperlinked_grep",
+		Run: func(cmd *cli.Command, args []string) (int, error) {
+			opts := Options{}
+			err := cmd.GetOptionValues(&opts)
+			if err != nil {
+				return 1, err
+			}
+			return run_func(cmd, &opts, args)
+		},
+		Hidden: true,
+	})
+	specialize_command(ans)
+	clone := root.AddClone(ans.Group, ans)
+	clone.Hidden = false
+	clone.Name = "hyperlinked-grep"
+}
+
 func EntryPoint(parent *cli.Command) {
 	create_cmd(parent, main)
 }
