@@ -42,7 +42,9 @@ Mirror the image about a horizontal or vertical axis or both.
 
 --clear
 type=bool-set
-Remove all images currently displayed on the screen.
+Remove all images currently displayed on the screen. Note that
+this cannot work with terminal multiplexers such as tmux since
+only the multiplexer can know the position of the screen.
 
 
 --transfer-mode
@@ -133,7 +135,7 @@ Use the Unicode placeholder method to display the images. Useful to display
 images from within full screen terminal programs that do not understand the
 kitty graphics protocol such as multiplexers or editors. See
 :ref:`graphics_unicode_placeholders` for details. Note that when using this
-method, placed (with :option:`--place`) images that do not fit on the screen,
+method, images placed (with :option:`--place`) that do not fit on the screen,
 will get wrapped at the screen edge instead of getting truncated. This
 wrapping is per line and therefore the image will look like it is interleaved
 with blank lines.
@@ -155,6 +157,12 @@ default=0
 The graphics protocol id to use for the created image. Normally, a random id is created if needed.
 This option allows control of the id. When multiple images are sent, sequential ids starting from the specified id
 are used. Valid ids are from 1 to 4294967295. Numbers outside this range are automatically wrapped.
+
+
+--no-trailing-newline -n
+type=bool-set
+By default, the cursor is moved to the next line after displaying an image. This option, prevents that. Should not be used
+when catting multiple images. Also has no effect when the :option:`--place` option is used.
 '''
 
 help_text = (
@@ -173,7 +181,7 @@ if __name__ == '__main__':
 elif __name__ == '__doc__':
     import sys
 
-    from kitty.cli import CompletionSpec
+    from kitty.simple_cli_definitions import CompletionSpec
     cd = sys.cli_docs  # type: ignore
     cd['usage'] = usage
     cd['options'] = lambda: OPTIONS.format()
