@@ -4,13 +4,15 @@ package icat
 
 import (
 	"fmt"
+	"image"
+	"image/gif"
+
+	"github.com/kovidgoyal/go-parallel"
 	"github.com/kovidgoyal/kitty/tools/tty"
 	"github.com/kovidgoyal/kitty/tools/tui/graphics"
 	"github.com/kovidgoyal/kitty/tools/utils"
 	"github.com/kovidgoyal/kitty/tools/utils/images"
 	"github.com/kovidgoyal/kitty/tools/utils/shm"
-	"image"
-	"image/gif"
 
 	"github.com/kovidgoyal/exiffix"
 	"github.com/kovidgoyal/imaging"
@@ -49,13 +51,13 @@ func add_frame(ctx *images.Context, imgd *image_data, img image.Image) *image_fr
 	bytes_per_pixel := 4
 
 	if is_opaque || remove_alpha != nil {
-		var rgb *images.NRGB
+		var rgb *imaging.NRGB
 		bytes_per_pixel = 3
 		m, err := shm.CreateTemp(shm_template, uint64(f.width*f.height*bytes_per_pixel))
 		if err != nil {
-			rgb = images.NewNRGB(dest_rect)
+			rgb = imaging.NewNRGB(dest_rect)
 		} else {
-			rgb = &images.NRGB{Pix: m.Slice(), Stride: bytes_per_pixel * f.width, Rect: dest_rect}
+			rgb = &imaging.NRGB{Pix: m.Slice(), Stride: bytes_per_pixel * f.width, Rect: dest_rect}
 			f.shm = m
 		}
 		f.transmission_format = graphics.GRT_format_rgb
@@ -151,6 +153,11 @@ func add_gif_frames(ctx *images.Context, imgd *image_data, gf *gif.GIF) error {
 }
 
 func render_image_with_go(imgd *image_data, src *opened_input) (err error) {
+	defer func() {
+		if r := recover(); r != nil {
+			err = parallel.Format_stacktrace_on_panic(r, 1)
+		}
+	}()
 	ctx := images.Context{}
 	switch {
 	case imgd.format_uppercase == "GIF" && opts.Loop != 0:
