@@ -11,10 +11,10 @@ import (
 	"strings"
 	"time"
 
-	"kitty/tools/cli"
-	"kitty/tools/themes"
-	"kitty/tools/tui/loop"
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/themes"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 var _ = fmt.Print
@@ -111,12 +111,12 @@ func parse_theme_metadata() error {
 	for _, path := range paths {
 		if path != "" {
 			metadata, _, err := themes.ParseThemeMetadata(path)
-			if metadata.Name == "" {
-				metadata.Name = themes.ThemeNameFromFileName(filepath.Base(path))
-			}
 			if err != nil {
 				return err
 			}
+			if metadata.Name == "" {
+				metadata.Name = themes.ThemeNameFromFileName(filepath.Base(path))
+			}
 			ans = append(ans, metadata)
 		}
 	}
