@@ -84,7 +84,7 @@ gl_init(void) {
     }
 }
 
-static const char*
+const char*
 check_framebuffer_status(void) {
     GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
     switch (status) {
@@ -100,12 +100,6 @@ check_framebuffer_status(void) {
     }
 }
 
-void
-check_framebuffer_status_or_die(void) {
-    const char *err = check_framebuffer_status();
-    if (err != NULL) fatal("Framebuffer not complete with error: %s", err);
-}
-
 void
 free_texture(GLuint *tex_id) {
     glDeleteTextures(1, tex_id);
