@@ -7,7 +7,6 @@
 #pragma once
 #include "data-types.h"
 #include "monotonic.h"
-#include "kitty-uthash.h"
 
 typedef struct {
     unsigned char action, transmission_type, compressed, delete_action;
@@ -30,6 +29,23 @@ typedef struct {
     float left, top, right, bottom;
 } ImageRect;
 
+typedef struct {
+    ImageRect src_rect, dest_rect;
+    uint32_t texture_id, group_count;
+    int z_index;
+    id_type image_id, ref_id;
+} ImageRenderData;
+
+typedef struct {
+    uint32_t texture_id;
+    unsigned int height, width;
+    uint8_t* bitmap;
+    uint32_t refcnt, id;
+    size_t mmap_size;
+} BackgroundImage;
+
+
+#ifdef GRAPHICS_INTERNAL_APIS
 typedef struct {
     float src_width, src_height, src_x, src_y;
     uint32_t cell_x_offset, cell_y_offset, num_cols, num_rows, effective_num_rows, effective_num_cols;
@@ -51,7 +67,6 @@ typedef struct {
     } parent;
 
     id_type internal_id;
-    hash_handle_type hh;
 } ImageRef;
 
 typedef struct {
@@ -65,13 +80,17 @@ typedef struct TextureRef {
     uint32_t id, refcnt;
 } TextureRef;
 
+#define NAME ref_map
+#define KEY_TY id_type
+#define VAL_TY ImageRef*
+#include "kitty-verstable.h"
+
 typedef struct {
     uint32_t client_id, client_number, width, height;
     TextureRef *texture;
     id_type internal_id;
 
     bool root_frame_data_loaded;
-    ImageRef *refs;
     id_type ref_id_counter;
     Frame *extra_frames, root_frame;
     uint32_t current_frame_index, frame_id_counter;
@@ -83,24 +102,9 @@ typedef struct {
     AnimationState animation_state;
     uint32_t max_loops, current_loop;
     monotonic_t current_frame_shown_at;
-
-    hash_handle_type hh;
+    ref_map refs_by_internal_id;
 } Image;
 
-typedef struct {
-    uint32_t texture_id;
-    unsigned int height, width;
-    uint8_t* bitmap;
-    uint32_t refcnt;
-} BackgroundImage;
-
-typedef struct {
-    ImageRect src_rect, dest_rect;
-    uint32_t texture_id, group_count;
-    int z_index;
-    id_type image_id, ref_id;
-} ImageRenderData;
-
 typedef struct {
     id_type image_id;
     uint32_t frame_id;
@@ -122,12 +126,16 @@ typedef struct {
     ImageAndFrame loading_for;
 } LoadData;
 
+#define NAME image_map
+#define KEY_TY id_type
+#define VAL_TY Image*
+#include "kitty-verstable.h"
+
 typedef struct {
     PyObject_HEAD
 
     size_t storage_limit;
     LoadData currently_loading;
-    Image *images;
     id_type image_id_counter;
     struct {
         size_t count, capacity;
@@ -141,8 +149,11 @@ typedef struct {
     PyObject *disk_cache;
     bool has_images_needing_animation, context_made_current_for_this_command;
     id_type window_id;
+    image_map images_by_internal_id;
 } GraphicsManager;
-
+#else
+typedef struct {int x;} *GraphicsManager;
+#endif
 
 typedef struct {
     int32_t amt, limit;
@@ -159,31 +170,38 @@ gl_size(const unsigned int sz, const unsigned int viewport_size) {
 }
 
 static inline float
-clamp_position_to_nearest_pixel(float pos, const unsigned int viewport_size) {
-    // clamp the specified opengl position to the nearest pixel
-    const float px = 2.f / viewport_size;
-    const float distance =  pos + 1.f;
-    const float num_of_pixels = roundf(distance / px);
-    return -1.f + num_of_pixels * px;
-}
-
-static inline float
-gl_pos_x(const unsigned int px_from_left_margin, const unsigned int viewport_size) {
+gl_pos_x(const int px_from_left_margin, const unsigned int viewport_size) {
     const float px = 2.f / viewport_size;
     return -1.f + px_from_left_margin * px;
 }
 
 static inline float
-gl_pos_y(const unsigned int px_from_top_margin, const unsigned int viewport_size) {
+tex_pos_x(const int px_from_left_margin, const unsigned texture_width) {
+    return px_from_left_margin / (float)texture_width;
+}
+
+static inline float
+gl_pos_y(const int px_from_top_margin, const unsigned int viewport_size) {
     const float px = 2.f / viewport_size;
     return 1.f - px_from_top_margin * px;
 }
 
+static inline float
+tex_pos_y(const int px_from_top_margin, const unsigned texture_height) {
+    const int px_from_bottom_margin = texture_height - px_from_top_margin;
+    return px_from_bottom_margin / (float)texture_height;
+}
+
+
+typedef struct GraphicsRenderData {
+    size_t count, capacity, num_of_below_refs, num_of_negative_refs, num_of_positive_refs;
+    ImageRenderData *images;
+} GraphicsRenderData;
 
 GraphicsManager* grman_alloc(bool for_paused_rendering);
 void grman_clear(GraphicsManager*, bool, CellPixelSize fg);
 const char* grman_handle_command(GraphicsManager *self, const GraphicsCommand *g, const uint8_t *payload, Cursor *c, bool *is_dirty, CellPixelSize fg);
-Image* grman_put_cell_image(GraphicsManager *self, uint32_t row, uint32_t col, uint32_t image_id, uint32_t placement_id, uint32_t x, uint32_t y, uint32_t w, uint32_t h, CellPixelSize cell);
+void grman_put_cell_image(GraphicsManager *self, uint32_t row, uint32_t col, uint32_t image_id, uint32_t placement_id, uint32_t x, uint32_t y, uint32_t w, uint32_t h, CellPixelSize cell);
 bool grman_update_layers(GraphicsManager *self, unsigned int scrolled_by, float screen_left, float screen_top, float dx, float dy, unsigned int num_cols, unsigned int num_rows, CellPixelSize);
 void grman_scroll_images(GraphicsManager *self, const ScrollData*, CellPixelSize fg);
 void grman_resize(GraphicsManager*, index_type, index_type, index_type, index_type, index_type, index_type);
@@ -194,6 +212,10 @@ void gpu_data_for_image(ImageRenderData *ans, float left, float top, float right
 bool png_from_file_pointer(FILE* fp, const char *path, uint8_t** data, unsigned int* width, unsigned int* height, size_t* sz);
 bool png_path_to_bitmap(const char *path, uint8_t** data, unsigned int* width, unsigned int* height, size_t* sz);
 bool png_from_data(void *png_data, size_t png_data_sz, const char *path_for_error_messages, uint8_t** data, unsigned int* width, unsigned int* height, size_t* sz);
+bool image_path_to_bitmap(const char *path, uint8_t** data, unsigned int* width, unsigned int* height, size_t* sz);
 bool scan_active_animations(GraphicsManager *self, const monotonic_t now, monotonic_t *minimum_gap, bool os_window_context_set);
-void scale_rendered_graphic(ImageRenderData*, float xstart, float ystart, float x_scale, float y_scale);
 void grman_pause_rendering(GraphicsManager *self, GraphicsManager *dest);
+void grman_mark_layers_dirty(GraphicsManager *self);
+void grman_set_window_id(GraphicsManager *self, id_type id);
+bool grman_has_images(GraphicsManager *self);
+GraphicsRenderData grman_render_data(GraphicsManager *self);
