@@ -7,7 +7,7 @@ from itertools import repeat
 from typing import Any, Callable, NamedTuple
 
 from kitty.borders import BorderColor
-from kitty.fast_data_types import Region, set_active_window, viewport_for_window
+from kitty.fast_data_types import Region, get_options, set_active_window, viewport_for_window
 from kitty.options.types import Options
 from kitty.types import Edges, WindowGeometry, WindowMapper
 from kitty.typing_compat import TypedDict, WindowType
@@ -69,8 +69,15 @@ def idx_for_id(win_id: int, windows: Iterable[WindowType]) -> int | None:
     return None
 
 
+def effective_draw_minimal_borders(opts: Options, has_more_than_one_visible_group: bool = True) -> bool:
+    ans = opts.draw_minimal_borders and sum(opts.window_margin_width) == 0
+    if not has_more_than_one_visible_group and opts.draw_window_borders_for_single_window:
+        ans = False
+    return ans
+
+
 def set_layout_options(opts: Options) -> None:
-    lgd.draw_minimal_borders = opts.draw_minimal_borders and sum(opts.window_margin_width) == 0
+    lgd.draw_minimal_borders = effective_draw_minimal_borders(opts)
     lgd.draw_active_borders = opts.active_border_color is not None
     lgd.alignment_x = -1 if opts.placement_strategy.endswith('left') else 1 if opts.placement_strategy.endswith('right') else 0
     lgd.alignment_y = -1 if opts.placement_strategy.startswith('top') else 1 if opts.placement_strategy.startswith('bottom') else 0
@@ -248,7 +255,7 @@ class Layout:
         self.set_active_window_in_os_window = partial(set_active_window, os_window_id, tab_id)
 
     def bias_increment_for_cell(self, all_windows: WindowList, is_horizontal: bool) -> float:
-        self._set_dimensions()
+        self._set_dimensions(all_windows)
         return self.calculate_bias_increment_for_a_single_cell(all_windows, is_horizontal)
 
     def calculate_bias_increment_for_a_single_cell(self, all_windows: WindowList, is_horizontal: bool) -> float:
@@ -335,7 +342,7 @@ class Layout:
         if bias is not None:
             idx = all_windows.group_idx_for_window(window)
             if idx is not None:
-                self._set_dimensions()
+                self._set_dimensions(all_windows)
                 self._bias_slot(all_windows, idx, bias)
 
     def _bias_slot(self, all_windows: WindowList, idx: int, bias: float) -> bool:
@@ -354,11 +361,15 @@ class Layout:
             is_visible = window is active_window or (is_group_leader and not self.only_active_window_visible)
             window.set_visible_in_layout(is_visible)
 
-    def _set_dimensions(self) -> None:
+    def _set_dimensions(self, all_windows: WindowList) -> None:
         lgd.central, tab_bar, vw, vh, lgd.cell_width, lgd.cell_height = viewport_for_window(self.os_window_id)
+        # Update lgd.draw_minimal_borders based on the current number of visible windows
+        # and the draw_window_borders_for_single_window option
+        opts = get_options()
+        lgd.draw_minimal_borders = effective_draw_minimal_borders(opts, all_windows.has_more_than_one_visible_group)
 
     def __call__(self, all_windows: WindowList) -> None:
-        self._set_dimensions()
+        self._set_dimensions(all_windows)
         self.update_visibility(all_windows)
         self.blank_rects = []
         self.do_layout(all_windows)
@@ -432,7 +443,7 @@ class Layout:
         return all_windows.compute_needs_borders_map(lgd.draw_active_borders)
 
     def get_minimal_borders(self, windows: WindowList) -> Generator[BorderLine, None, None]:
-        self._set_dimensions()
+        self._set_dimensions(windows)
         yield from self.minimal_borders(windows)
 
     def minimal_borders(self, windows: WindowList) -> Generator[BorderLine, None, None]:
