@@ -67,6 +67,7 @@ from .utils import (
     get_custom_window_icon,
     log_error,
     parse_os_window_state,
+    read_shell_environment,
     safe_mtime,
     startup_notification_handler,
 )
@@ -191,7 +192,8 @@ def set_cocoa_global_shortcuts(opts: Options) -> dict[str, SingleKey]:
                 func_map[parts].append(single_key)
 
         for ac in ('new_os_window', 'close_os_window', 'close_tab', 'edit_config_file', 'previous_tab',
-                   'next_tab', 'new_tab', 'new_window', 'close_window', 'toggle_macos_secure_keyboard_entry', 'toggle_fullscreen',
+                   'next_tab', 'new_tab', 'new_window', 'close_window', 'toggle_macos_secure_keyboard_entry',
+                   'toggle_fullscreen', 'macos_cycle_through_os_windows',
                    'hide_macos_app', 'hide_macos_other_apps', 'minimize_macos_window', 'quit'):
             val = get_macos_shortcut_for(func_map, ac)
             if val is not None:
@@ -478,6 +480,17 @@ def setup_environment(opts: Options, cli_opts: CLIOptions) -> None:
         from_config_file = True
     if cli_opts.listen_on:
         cli_opts.listen_on = expand_listen_on(cli_opts.listen_on, from_config_file)
+    if vars := opts.env.pop('read_from_shell', ''):
+        import fnmatch
+        import re
+        senv = read_shell_environment(opts)
+        patterns = tuple(re.compile(fnmatch.translate(x.strip())) for x in vars.split() if x.strip())
+        if patterns:
+            for k, v in senv.items():
+                for pat in patterns:
+                    if pat.match(k) is not None:
+                        opts.env[k] = v
+                        break
     env = opts.env.copy()
     ensure_kitty_in_path()
     ensure_kitten_in_path()
