@@ -6,16 +6,26 @@
 
 #pragma once
 
-#include "data-types.h"
+#include <stdint.h>
+#include <stdbool.h>
 #include <png.h>
-typedef void(*png_error_handler_func)(const char*, const char*);
-typedef struct {
+
+typedef struct png_read_data png_read_data;
+
+typedef void(*png_error_handler_func)(png_read_data *d, const char*, const char*);
+
+typedef struct png_read_data {
     uint8_t *decompressed;
     bool ok;
     png_bytep *row_pointers;
     int width, height;
     size_t sz;
     png_error_handler_func err_handler;
+    struct {
+        char *buf;
+        size_t used, capacity;
+    } error;
 } png_read_data;
 
 void inflate_png_inner(png_read_data *d, const uint8_t *buf, size_t bufsz);
+const char* png_from_32bit_rgba(uint32_t *data, size_t width, size_t height, size_t *out_size, bool flip_vertically);
