@@ -2,12 +2,9 @@
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
 
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
-from .base import (
-    MATCH_TAB_OPTION, ArgsType, Boss, PayloadGetType, PayloadType, RCOptions,
-    RemoteCommand, ResponseType, Window
-)
+from .base import MATCH_TAB_OPTION, ArgsType, Boss, PayloadGetType, PayloadType, RCOptions, RemoteCommand, ResponseType, Window
 
 if TYPE_CHECKING:
     from kitty.cli_stub import FocusTabRCOptions as CLIOptions
@@ -15,7 +12,7 @@ if TYPE_CHECKING:
 
 class FocusTab(RemoteCommand):
 
-    '''
+    protocol_spec = __doc__ = '''
     match/str: The tab to focus
     '''
 
@@ -29,15 +26,18 @@ default=false
 Don't wait for a response indicating the success of the action. Note that
 using this option means that you will not be notified of failures.
 '''
-    argspec = ''
 
     def message_to_kitty(self, global_opts: RCOptions, opts: 'CLIOptions', args: ArgsType) -> PayloadType:
-        return {'match': opts.match}
+        return {'match': opts.match, 'no_response': opts.no_response}
 
-    def response_from_kitty(self, boss: Boss, window: Optional[Window], payload_get: PayloadGetType) -> ResponseType:
+    def response_from_kitty(self, boss: Boss, window: Window | None, payload_get: PayloadGetType) -> ResponseType:
         for tab in self.tabs_for_match_payload(boss, window, payload_get):
             if tab:
-                boss.set_active_tab(tab)
+                window = tab.active_window
+                if window:
+                    boss.set_active_window(window, switch_os_window_if_needed=True)
+                else:
+                    boss.set_active_tab(tab)
                 break
         return None
 
