@@ -4,8 +4,6 @@
 
 from typing import TYPE_CHECKING
 
-from kitty.fast_data_types import focus_os_window
-
 from .base import MATCH_WINDOW_OPTION, ArgsType, Boss, PayloadGetType, PayloadType, RCOptions, RemoteCommand, ResponseType, Window
 
 if TYPE_CHECKING:
@@ -33,9 +31,7 @@ the command will exit with a success code.
     def response_from_kitty(self, boss: Boss, window: Window | None, payload_get: PayloadGetType) -> ResponseType:
         for window in self.windows_for_match_payload(boss, window, payload_get):
             if window:
-                os_window_id = boss.set_active_window(window)
-                if os_window_id:
-                    focus_os_window(os_window_id, True)
+                boss.set_active_window(window, switch_os_window_if_needed=True)
                 break
         return None
 
