@@ -1,20 +1,21 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
-import imghdr
 import os
 from base64 import standard_b64decode, standard_b64encode
-from typing import TYPE_CHECKING, Optional
+from io import BytesIO
+from typing import TYPE_CHECKING
 
 from kitty.types import AsyncResponse
+from kitty.utils import is_png
 
 from .base import (
     MATCH_WINDOW_OPTION,
+    SUPPORTED_IMAGE_FORMATS,
     ArgsType,
     Boss,
     CmdGenerator,
     ImageCompletion,
-    NamedTemporaryFile,
     PayloadGetType,
     PayloadType,
     RCOptions,
@@ -43,10 +44,10 @@ class SetBackgroundImage(RemoteCommand):
 
     short_desc = 'Set the background image'
     desc = (
-        'Set the background image for the specified OS windows. You must specify the path to a PNG image that'
+        'Set the background image for the specified OS windows. You must specify the path to an image that'
         ' will be used as the background. If you specify the special value :code:`none` then any existing image will'
-        ' be removed.'
-    )
+        ' be removed. Supported image formats are: '
+    ) + ', '.join(SUPPORTED_IMAGE_FORMATS)
     options_spec = f'''\
 --all -a
 type=bool-set
@@ -80,16 +81,18 @@ failed, the command will exit with a success code.
         if len(args) != 1:
             self.fatal('Must specify path to exactly one PNG image')
         path = os.path.expanduser(args[0])
+        import secrets
         ret = {
             'match': opts.match,
             'configured': opts.configured,
             'layout': opts.layout,
             'all': opts.all,
+            'stream_id': secrets.token_urlsafe(),
         }
         if path.lower() == 'none':
             ret['data'] = '-'
             return ret
-        if imghdr.what(path) != 'png':
+        if not is_png(path):
             self.fatal(f'{path} is not a PNG image')
 
         def file_pipe(path: str) -> CmdGenerator:
@@ -104,24 +107,23 @@ failed, the command will exit with a success code.
             yield ret
         return file_pipe(path)
 
-    def response_from_kitty(self, boss: Boss, window: Optional[Window], payload_get: PayloadGetType) -> ResponseType:
+    def response_from_kitty(self, boss: Boss, window: Window | None, payload_get: PayloadGetType) -> ResponseType:
         data = payload_get('data')
-        windows = self.windows_for_payload(boss, window, payload_get)
+        windows = self.windows_for_payload(boss, window, payload_get, window_match_name='match')
         os_windows = tuple({w.os_window_id for w in windows if w})
         layout = payload_get('layout')
         if data == '-':
             path = None
-            tfile = NamedTemporaryFile()
+            tfile = BytesIO()
         else:
             q = self.handle_streamed_data(standard_b64decode(data) if data else b'', payload_get)
             if isinstance(q, AsyncResponse):
                 return q
-            path = q.name
+            path = '/image/from/remote/control'
             tfile = q
 
         try:
-            with tfile:
-                boss.set_background_image(path, os_windows, payload_get('configured'), layout)
+            boss.set_background_image(path, os_windows, payload_get('configured'), layout, tfile.getvalue())
         except ValueError as err:
             err.hide_traceback = True  # type: ignore
             raise
