@@ -24,7 +24,7 @@ typedef enum { LEFT_EDGE = 1, TOP_EDGE = 2, RIGHT_EDGE = 4, BOTTOM_EDGE = 8 } Ed
 typedef enum { REPEAT_MIRROR, REPEAT_CLAMP, REPEAT_DEFAULT } RepeatStrategy;
 typedef enum { WINDOW_NORMAL, WINDOW_FULLSCREEN, WINDOW_MAXIMIZED, WINDOW_MINIMIZED, WINDOW_HIDDEN } WindowState;
 
-typedef struct {
+typedef struct UrlPrefix {
     char_type string[16];
     size_t len;
 } UrlPrefix;
@@ -38,7 +38,7 @@ struct MenuItem {
     const char *definition;
 };
 
-typedef struct {
+typedef struct Options {
     monotonic_t visual_bell_duration, cursor_blink_interval, cursor_stop_blinking_after, click_interval;
     struct {
         monotonic_t hide_wait, unhide_wait;
@@ -54,6 +54,7 @@ typedef struct {
     monotonic_t cursor_trail;
     float cursor_trail_decay_fast;
     float cursor_trail_decay_slow;
+    color_type cursor_trail_color;
     float cursor_trail_start_threshold;
     unsigned int url_style;
     unsigned int scrollback_pager_history_size;
@@ -69,7 +70,14 @@ typedef struct {
     float macos_thicken_font;
     WindowTitleIn macos_show_window_title_in;
     char *bell_path, *bell_theme;
-    float background_opacity, dim_opacity, scrollback_indicator_opacity;
+    float background_opacity, dim_opacity;
+
+    ScrollbarVisibilityPolicy scrollbar;
+    bool scrollbar_interactive, scrollbar_jump_on_click;
+    float scrollbar_width, scrollbar_radius, scrollbar_gap, scrollbar_min_handle_height, scrollbar_hitbox_expansion;
+    float scrollbar_hover_width, scrollbar_handle_opacity, scrollbar_track_opacity, scrollbar_track_hover_opacity;
+    color_type scrollbar_handle_color, scrollbar_track_color;
+
     float text_contrast, text_gamma_adjustment;
     bool text_old_gamma;
 
@@ -142,16 +150,19 @@ typedef struct WindowLogoRenderData {
 } WindowLogoRenderData;
 
 typedef struct {
+    unsigned int left, top, right, bottom;
+    struct {
+        unsigned int left, top, right, bottom;
+    } spaces;
+} WindowGeometry;
+
+typedef struct WindowRenderData {
     ssize_t vao_idx;
-    float xstart, ystart, dx, dy;
+    WindowGeometry geometry;
     Screen *screen;
 } WindowRenderData;
 
-typedef struct {
-    unsigned int left, top, right, bottom;
-} WindowGeometry;
-
-typedef struct {
+typedef struct Click {
     monotonic_t at;
     int button, modifiers;
     double x, y;
@@ -159,7 +170,7 @@ typedef struct {
 } Click;
 
 #define CLICK_QUEUE_SZ 3
-typedef struct {
+typedef struct ClickQueue {
     Click clicks[CLICK_QUEUE_SZ];
     unsigned int length;
 } ClickQueue;
@@ -189,11 +200,9 @@ typedef struct WindowBarData {
     bool needs_render;
 } WindowBarData;
 
-typedef struct {
+typedef struct Window {
     id_type id;
     bool visible;
-    float cursor_opacity_at_last_render;
-    CursorShape last_cursor_shape;
     PyObject *title;
     WindowRenderData render_data;
     WindowLogoRenderData window_logo;
@@ -201,7 +210,6 @@ typedef struct {
     struct {
         unsigned int left, top, right, bottom;
     } padding;
-    WindowGeometry geometry;
     ClickQueue click_queues[8];
     monotonic_t last_drag_scroll_at;
     uint32_t last_special_key_pressed;
@@ -216,21 +224,28 @@ typedef struct {
         PendingClick *clicks;
         size_t num, capacity;
     } pending_clicks;
+    struct {
+        double thumb_top, thumb_bottom;
+        bool is_dragging;
+        double drag_start_y;
+        double drag_start_scrolled_by;
+        bool is_hovering;
+    } scrollbar;
 } Window;
 
-typedef struct {
+typedef struct BorderRect {
     float left, top, right, bottom;
     uint32_t color;
 } BorderRect;
 
-typedef struct {
+typedef struct BorderRects {
     BorderRect *rect_buf;
     unsigned int num_border_rects, capacity;
     bool is_dirty;
     ssize_t vao_idx;
 } BorderRects;
 
-typedef struct {
+typedef struct CursorTrail {
     bool needs_render;
     monotonic_t updated_at;
     float opacity;
@@ -240,7 +255,7 @@ typedef struct {
     float cursor_edge_y[2];
 } CursorTrail;
 
-typedef struct {
+typedef struct Tab {
     id_type id;
     unsigned int active_window, num_windows, capacity;
     Window *windows;
@@ -251,7 +266,7 @@ typedef struct {
 enum RENDER_STATE { RENDER_FRAME_NOT_REQUESTED, RENDER_FRAME_REQUESTED, RENDER_FRAME_READY };
 typedef enum { NO_CLOSE_REQUESTED, CONFIRMABLE_CLOSE_REQUESTED, CLOSE_BEING_CONFIRMED, IMPERATIVE_CLOSE_REQUESTED } CloseRequest;
 
-typedef struct {
+typedef struct LiveResizeInfo {
     monotonic_t last_resize_event_at;
     bool in_progress;
     bool from_os_notification;
@@ -271,7 +286,14 @@ typedef struct WindowChromeState {
     float background_opacity;
 } WindowChromeState;
 
-typedef struct {
+typedef struct BackgroundImageRenderSettings {
+    struct { unsigned width, height; } os_window;
+    unsigned instance_id;
+    BackgroundImageLayout layout;
+    bool linear; uint32_t bgcolor; float opacity;
+} BackgroundImageRenderSettings;
+
+typedef struct OSWindow {
     void *handle;
     id_type id;
     monotonic_t created_at;
@@ -283,8 +305,12 @@ typedef struct {
     double viewport_x_ratio, viewport_y_ratio;
     Tab *tabs;
     BackgroundImage *bgimage;
+    struct {
+        uint32_t texture_id, framebuffer_id;
+        int width, height;
+    } indirect_output;
     unsigned int active_tab, num_tabs, capacity, last_active_tab, last_num_tabs, last_active_window_id;
-    bool focused_at_last_render, needs_render;
+    bool focused_at_last_render, needs_render, needs_layers;
     unsigned keep_rendering_till_swap;
     WindowRenderData tab_bar_render_data;
     struct {
@@ -302,10 +328,10 @@ typedef struct {
     bool viewport_size_dirty, viewport_updated_at_least_once;
     monotonic_t viewport_resized_at;
     LiveResizeInfo live_resize;
-    bool has_pending_resizes, is_semi_transparent, shown_once, ignore_resize_events;
-    unsigned int clear_count, redraw_count;
+    bool has_pending_resizes, shown_once, ignore_resize_events;
+    unsigned int redraw_count;
     WindowChromeState last_window_chrome;
-    float background_opacity;
+    struct { float alpha; bool os_forces_opaque, supports_transparency; } background_opacity;
     FONTS_DATA_HANDLE fonts_data;
     id_type temp_font_group_id;
     enum RENDER_STATE render_state;
@@ -316,8 +342,13 @@ typedef struct {
     bool is_layer_shell, hide_on_focus_loss;
 } OSWindow;
 
+static inline float
+effective_os_window_alpha(OSWindow *w) {
+    return (!w->background_opacity.supports_transparency || w->background_opacity.os_forces_opaque) ?
+        1.f : w->background_opacity.alpha;
+}
 
-typedef struct {
+typedef struct GlobalState {
     Options opts;
 
     id_type os_window_id_counter, tab_id_counter, window_id_counter;
@@ -338,6 +369,7 @@ typedef struct {
     bool redirect_mouse_handling;
     WindowLogoTable *all_window_logos;
     int gl_version;
+    bool supports_framebuffer_srgb;
     PyObject *options_object;
 } GlobalState;
 
@@ -370,7 +402,6 @@ void mark_os_window_for_close(OSWindow* w, CloseRequest cr);
 void update_os_window_viewport(OSWindow *window, bool notify_boss);
 bool should_os_window_be_rendered(OSWindow* w);
 void wakeup_main_loop(void);
-void swap_window_buffers(OSWindow *w);
 bool make_window_context_current(id_type);
 void hide_mouse(OSWindow *w);
 bool is_mouse_hidden(OSWindow *w);
@@ -384,21 +415,19 @@ OSWindow* add_os_window(void);
 OSWindow* current_os_window(void);
 void os_window_regions(OSWindow*, Region *main, Region *tab_bar);
 bool drag_scroll(Window *, OSWindow*);
-void draw_borders(ssize_t vao_idx, unsigned int num_border_rects, BorderRect *rect_buf, bool rect_data_is_dirty, uint32_t viewport_width, uint32_t viewport_height, color_type, unsigned int, bool, OSWindow *w);
+void draw_borders(ssize_t vao_idx, unsigned int num_border_rects, BorderRect *rect_buf, bool rect_data_is_dirty, color_type, unsigned int, bool, OSWindow *w);
 ssize_t create_cell_vao(void);
 ssize_t create_graphics_vao(void);
 ssize_t create_border_vao(void);
-bool send_cell_data_to_gpu(ssize_t, float, float, float, float, Screen *, OSWindow *);
-void draw_cells(ssize_t, const WindowRenderData*, OSWindow *, bool, bool, bool, Window*);
-void draw_centered_alpha_mask(OSWindow *w, size_t screen_width, size_t screen_height, size_t width, size_t height, uint8_t *canvas, float);
-void draw_cursor_trail(CursorTrail *trail, Window *active_window);
+bool send_cell_data_to_gpu(ssize_t, Screen *, OSWindow *);
+void draw_cells(const WindowRenderData*, OSWindow *, bool, bool, bool, Window*);
 bool update_cursor_trail(CursorTrail *ct, Window *w, monotonic_t now, OSWindow *os_window);
-void update_surface_size(int, int, uint32_t);
+void set_gpu_viewport(unsigned w, unsigned h);
 void free_texture(uint32_t*);
 void free_framebuffer(uint32_t*);
 void send_image_to_gpu(uint32_t*, const void*, int32_t, int32_t, bool, bool, bool, RepeatStrategy);
 void send_sprite_to_gpu(FONTS_DATA_HANDLE fg, sprite_index, pixel*, sprite_index);
-void blank_canvas(float, color_type);
+void blank_canvas(float, color_type, bool);
 void blank_os_window(OSWindow *);
 void set_os_window_chrome(OSWindow *w);
 FONTS_DATA_HANDLE load_fonts_data(double, double, double);
@@ -429,7 +458,7 @@ const char* format_mods(unsigned mods);
 void dispatch_pending_clicks(id_type, void*);
 void send_pending_click_to_window(Window*, int);
 void get_platform_dependent_config_values(void *glfw_window);
-bool draw_window_title(OSWindow *window, const char *text, color_type fg, color_type bg, uint8_t *output_buf, size_t width, size_t height);
+bool draw_window_title(double, double, const char *text, color_type fg, color_type bg, uint8_t *output_buf, size_t width, size_t height);
 uint8_t* draw_single_ascii_char(const char ch, size_t *result_width, size_t *result_height);
 bool is_os_window_fullscreen(OSWindow *);
 void update_ime_focus(OSWindow* osw, bool focused);
@@ -442,3 +471,6 @@ bool render_os_window(OSWindow *w, monotonic_t now, bool scan_for_animated_image
 void update_mouse_pointer_shape(void);
 void adjust_window_size_for_csd(OSWindow *w, int width, int height, int *adjusted_width, int *adjusted_height);
 void dispatch_buffered_keys(Window *w);
+bool screen_needs_rendering_in_layers(OSWindow *os_window, Window *w, Screen *screen);
+void setup_os_window_for_rendering(OSWindow*, Tab*, Window*, bool);
+void swap_window_buffers(OSWindow *w);
