@@ -3,7 +3,8 @@
 
 import re
 from binascii import hexlify, unhexlify
-from typing import Dict, Generator, Optional, cast
+from collections.abc import Generator
+from typing import Literal, cast
 
 from kitty.options.types import Options
 
@@ -33,6 +34,11 @@ termcap_aliases = {
 bool_capabilities = {
     # auto_right_margin (terminal has automatic margins)
     'am',
+    # auto_left_margin (cursor wraps on CUB1 from 0 to last column on prev line). This prevents ncurses
+    # from using BS (backspace) to position the cursor. See https://github.com/kovidgoyal/kitty/issues/8841
+    # It also allows using backspace with multi-line edits in cooked mode. Foot
+    # is the only other modern terminal I know of that implements this.
+    'bw',
     # can_change (terminal can redefine existing colors)
     'ccc',
     # has_meta key (i.e. sets the eight bit)
@@ -68,6 +74,7 @@ bool_capabilities = {
 
 termcap_aliases.update({
     'am': 'am',
+    'bw': 'bw',
     'cc': 'ccc',
     'km': 'km',
     '5i': 'mc5i',
@@ -283,7 +290,7 @@ string_capabilities = {
     'Cr': r'\E]112\007',
     # Indicates support for styled and colored underlines (non-standard) as
     # described at: https://sw.kovidgoyal.net/kitty/underlines/
-    # 'Setulc' is quivalent to the 'Su' boolean capability. Until
+    # 'Setulc' is equivalent to the 'Su' boolean capability. Until
     # standardized, specify both for application compatibility.
     'Setulc': r'\E[58:2:%p1%{65536}%/%d:%p1%{256}%/%{255}%&%d:%p1%{255}%&%d%;m',
 
@@ -484,7 +491,7 @@ termcap_aliases.update({
         'FV FW FX FY FZ Fa Fb Fc Fd Fe Ff Fg Fh Fi Fj Fk Fl Fm Fn Fo '
         'Fp Fq Fr'.split(), 1)})
 
-queryable_capabilities = cast(Dict[str, str], numeric_capabilities.copy())
+queryable_capabilities = cast(dict[str, str], numeric_capabilities.copy())
 queryable_capabilities.update(string_capabilities)
 extra = (bool_capabilities | numeric_capabilities.keys() | string_capabilities.keys()) - set(termcap_aliases.values())
 no_termcap_for = frozenset(
@@ -517,12 +524,14 @@ def key_as_bytes(name: str) -> bytes:
     return ans.encode('ascii')
 
 
-def get_capabilities(query_string: str, opts: 'Options') -> Generator[str, None, None]:
+def get_capabilities(query_string: str, opts: 'Options', window_id: int = 0, os_window_id: int = 0) -> Generator[str, None, None]:
     from .fast_data_types import ERROR_PREFIX
 
-    def result(encoded_query_name: str, x: Optional[str] = None) -> str:
+    def result(encoded_query_name: str, x: str | Literal[True] | None = None) -> str:
         if x is None:
             return f'0+r{encoded_query_name}'
+        if x is True:
+            return f'1+r{encoded_query_name}'
         return f'1+r{encoded_query_name}={hexlify(str(x).encode("utf-8")).decode("ascii")}'
 
     for encoded_query_name in query_string.split(';'):
@@ -532,7 +541,7 @@ def get_capabilities(query_string: str, opts: 'Options') -> Generator[str, None,
         elif name.startswith('kitty-query-'):
             from kittens.query_terminal.main import get_result
             name = name[len('kitty-query-'):]
-            rval = get_result(name)
+            rval = get_result(name, window_id, os_window_id)
             if rval is None:
                 from .utils import log_error
                 log_error('Unknown kitty terminfo query:', name)
@@ -540,6 +549,9 @@ def get_capabilities(query_string: str, opts: 'Options') -> Generator[str, None,
             else:
                 yield result(encoded_query_name, rval)
         else:
+            if name in bool_capabilities:
+                yield result(encoded_query_name, True)
+                continue
             try:
                 val = queryable_capabilities[name]
             except KeyError:
