@@ -137,10 +137,11 @@ def put_helpers(self, cw, ch, cols=10, lines=5):
         iid += 1
         imgid = kw.pop('id', None) or iid
         no_id = kw.pop('no_id', False)
+        a = kw.pop('a', 'T')
         if no_id:
-            cmd = 'a=T,f=24,s=%d,v=%d,%s' % (w, h, put_cmd(**kw))
+            cmd = f'a={a},f=24,s=%d,v=%d,%s' % (w, h, put_cmd(**kw))
         else:
-            cmd = 'a=T,f=24,i=%d,s=%d,v=%d,%s' % (imgid, w, h, put_cmd(**kw))
+            cmd = f'a={a},f=24,i=%d,s=%d,v=%d,%s' % (imgid, w, h, put_cmd(**kw))
         data = b'x' * w * h * 3
         res = send_command(screen, cmd, data)
         return imgid, parse_response(res)
@@ -233,32 +234,46 @@ class TestGraphics(BaseTest):
             for key, val in data.items():
                 self.ae(dc.get(key_as_bytes(key)), val)
 
-        def reset(small_hole_threshold=0):
+        def reset(small_hole_threshold=0, defrag_factor=2):
             nonlocal dc, data, s
             s = self.create_screen()
             dc = s.grman.disk_cache
             dc.small_hole_threshold = small_hole_threshold
+            dc.defrag_factor = defrag_factor
             data = {}
 
+        holes_to_create = 2, 4, 6, 8
         for i in range(25):
             self.assertIsNone(add(i, f'{i}' * i))
+            if i <= max(holes_to_create):
+                # We wait here to ensure data is written in order, otherwise the
+                # holes test below can fail
+                self.assertTrue(dc.wait_for_write())
 
         self.assertEqual(dc.total_size, sum(map(len, data.values())))
         self.assertTrue(dc.wait_for_write())
         check_data()
         sz = dc.size_on_disk()
         self.assertEqual(sz, sum(map(len, data.values())))
-        for x in (2, 4, 6, 8):
+        self.assertFalse(dc.holes())
+        holes = set()
+        for x in holes_to_create:
             remove(x)
+            holes.add(x)
             check_data()
             self.assertRaises(KeyError, dc.get, key_as_bytes(x))
             self.assertEqual(sz, dc.size_on_disk())
+            self.assertEqual(holes, {x[1] for x in dc.holes()})
         self.assertEqual(sz, dc.size_on_disk())
-        for x in ('xy', 'C'*4, 'B'*6, 'A'*8):
+        # fill holes largest first to ensure small one doesn't go into large accidentally causing fragmentation
+        for i, x in enumerate(sorted(holes, reverse=True)):
+            x = 'ABCDEFGH'[i] * x
             add(x, x)
             self.assertTrue(dc.wait_for_write())
-            self.assertEqual(sz, dc.size_on_disk())
             check_data()
+            holes.discard(len(x))
+            self.assertEqual(holes, {x[1] for x in dc.holes()})
+            self.assertEqual(sz, dc.size_on_disk(), f'Disk cache has unexpectedly grown from {sz} to {dc.size_on_disk} with data: {x!r}')
         check_data()
         dc.clear()
         st = time.monotonic()
@@ -307,27 +322,42 @@ class TestGraphics(BaseTest):
         dc.remove_from_ram(clear_predicate)
         self.assertEqual(dc.num_cached_in_ram(), 0)
 
-        reset(512)
+        reset(small_hole_threshold=512, defrag_factor=20)
         self.assertIsNone(add(1, '1' * 1024))
         self.assertIsNone(add(2, '2' * 1024))
         dc.wait_for_write()
         sz = dc.size_on_disk()
         remove(1)
         self.ae(sz, dc.size_on_disk())
+        self.ae({x[1] for x in dc.holes()}, {1024})
         self.assertIsNone(add(3, '3' * 800))
         dc.wait_for_write()
+        self.assertFalse(dc.holes())
         self.ae(sz, dc.size_on_disk())
         self.assertIsNone(add(4, '4' * 100))
         sz += 100
         dc.wait_for_write()
         self.ae(sz, dc.size_on_disk())
         check_data()
+        self.assertFalse(dc.holes())
         remove(4)
+        self.assertFalse(dc.holes())
         self.assertIsNone(add(5, '5' * 10))
         sz += 10
         dc.wait_for_write()
         self.ae(sz, dc.size_on_disk())
 
+        # test hole coalescing
+        reset(defrag_factor=20)
+        for i in range(1, 6):
+            self.assertIsNone(add(i, str(i)*i))
+            dc.wait_for_write()
+        remove(2)
+        remove(4)
+        self.assertEqual(dc.holes(), {(1, 2), (6, 4)})
+        remove(3)
+        self.assertEqual(dc.holes(), {(1, 9)})
+
     def test_suppressing_gr_command_responses(self):
         s, g, pl, sl = load_helpers(self)
         self.ae(pl('abcd', s=10, v=10, q=1), 'ENODATA:Insufficient image data: 4 < 400')
@@ -381,6 +411,17 @@ class TestGraphics(BaseTest):
         img = g.image_for_client_id(1)
         self.ae(img['data'], b'abcdefghijklmnop')
 
+        # Test interrupted and retried chunked load
+        self.assertIsNone(pl('abcd', s=2, v=2, m=1))
+        self.assertIsNone(pl('efgh', m=1))
+        send_command(s, 'a=d')  # delete command should clear partial transfer
+        self.assertIsNone(pl('abcd', s=2, v=2, m=1))
+        self.assertIsNone(pl('efgh', m=1))
+        self.assertIsNone(pl('ijkl', m=1))
+        self.ae(pl('1234', m=0), 'OK')
+        img = g.image_for_client_id(1)
+        self.ae(img['data'], b'abcdefghijkl1234')
+
         random_data = byte_block(32 * 1024)
         sl(
             random_data,
@@ -542,35 +583,50 @@ class TestGraphics(BaseTest):
         self.ae(s.grman.image_count, count - 1)
         delete(I=1)
         self.ae(s.grman.image_count, count - 2)
+        cn = 1117
+        li('abc', s=1, v=1, f=24, I=cn)
+        first_id = g.image_for_client_number(cn)['internal_id']
+        li('abc', s=1, v=1, f=24, I=cn)
+        second_id = g.image_for_client_number(cn)['internal_id']
+        self.assertNotEqual(first_id, second_id)
+        count = s.grman.image_count
+        delete(I=cn)
+        self.ae(g.image_for_client_number(cn)['internal_id'], first_id)
+        self.ae(s.grman.image_count, count - 1)
         s.reset()
         self.assertEqual(g.disk_cache.total_size, 0)
 
     def test_image_put(self):
         cw, ch = 10, 20
         s, dx, dy, put_image, put_ref, layers, rect_eq = put_helpers(self, cw, ch)
-        self.ae(put_image(s, 10, 20)[1], 'OK')
+        self.ae(put_image(s, cw, ch)[1], 'OK')
         l0 = layers(s)
         self.ae(len(l0), 1)
         rect_eq(l0[0]['src_rect'], 0, 0, 1, 1)
         rect_eq(l0[0]['dest_rect'], -1, 1, -1 + dx, 1 - dy)
         self.ae(l0[0]['group_count'], 1)
         self.ae(s.cursor.x, 1), self.ae(s.cursor.y, 0)
-        iid, (code, idstr) = put_ref(s, num_cols=s.columns, x_off=2, y_off=1, width=3, height=5, cell_x_off=3, cell_y_off=1, z=-1, placement_id=17)
+        src_width, src_height = 3, 5
+        iid, (code, idstr) = put_ref(s, num_cols=s.columns, num_lines=1, x_off=2, y_off=1, width=src_width, height=src_height,
+                                     cell_x_off=3, cell_y_off=1, z=-1, placement_id=17)
         self.ae(idstr, f'i={iid},p=17')
         l2 = layers(s)
         self.ae(len(l2), 2)
+        self.ae(l2[1], l0[0])
         rect_eq(l2[0]['src_rect'], 2 / 10, 1 / 20, (2 + 3) / 10, (1 + 5)/20)
-        left, top = -1 + dx + 3 * dx / cw, 1 - 1 * dy / ch
-        rect_eq(l2[0]['dest_rect'], left, top, -1 + (1 + s.columns) * dx, top - dy * 5 / ch)
-        rect_eq(l2[1]['src_rect'], 0, 0, 1, 1)
-        rect_eq(l2[1]['dest_rect'], -1, 1, -1 + dx, 1 - dy)
         self.ae(l2[0]['group_count'], 2)
-        self.ae(l2[1]['group_count'], 1)
+        left, top = -1 + dx + 3 * dx / cw, 1 - 1 * dy / ch
+        right = -1 + (1 + s.columns) * dx
+        bottom = 1 - dy
+        rect_eq(l2[0]['dest_rect'], left, top, right, bottom)
         self.ae(s.cursor.x, 0), self.ae(s.cursor.y, 1)
         self.ae(put_image(s, 10, 20, cursor_movement=1)[1], 'OK')
         self.ae(s.cursor.x, 0), self.ae(s.cursor.y, 1)
         s.reset()
         self.assertEqual(s.grman.disk_cache.total_size, 0)
+        self.ae(put_image(s, 2*cw, 2*ch, num_cols=3)[1], 'OK')
+        self.ae((s.cursor.x, s.cursor.y), (3, 2))
+        rect_eq(layers(s)[0]['dest_rect'], -1, 1, -1 + 3 * dx, 1 - 3*dy)
 
     def test_image_layer_grouping(self):
         cw, ch = 10, 20
@@ -710,7 +766,7 @@ class TestGraphics(BaseTest):
         s.draw("\U0010EEEE\u0305\u0305\U0010EEEE\u0305\u030D")
         # These two characters will be two separate refs (not contiguous).
         s.draw("\U0010EEEE\u0305\u0305\U0010EEEE\u0305\u030E")
-        s.cursor_back(4)
+        s.cursor_move(4)
         s.update_only_line_graphics_data()
         refs = layers(s)
         self.ae(len(refs), 3)
@@ -730,7 +786,7 @@ class TestGraphics(BaseTest):
         # The second image, 2x1
         s.apply_sgr("38;2;42;43;44")
         s.draw("\U0010EEEE\u0305\u030D\U0010EEEE\u0305\u030E")
-        s.cursor_back(2)
+        s.cursor_move(2)
         s.update_only_line_graphics_data()
         refs = layers(s)
         self.ae(len(refs), 2)
@@ -748,7 +804,7 @@ class TestGraphics(BaseTest):
         s.draw("\U0010EEEE\u0305\u0305\U0010EEEE\u0305\U0010EEEE\U0010EEEE\u0305")
         # full row 1 of the first image
         s.draw("\U0010EEEE\u030D\U0010EEEE\U0010EEEE\U0010EEEE\u030D\u0310")
-        s.cursor_back(8)
+        s.cursor_move(8)
         s.update_only_line_graphics_data()
         refs = layers(s)
         self.ae(len(refs), 2)
@@ -770,7 +826,7 @@ class TestGraphics(BaseTest):
         # This one will have id=43, which does not exist.
         s.apply_sgr("38;2;0;0;43")
         s.draw("\U0010EEEE\u0305\U0010EEEE\U0010EEEE\U0010EEEE")
-        s.cursor_back(4)
+        s.cursor_move(4)
         s.update_only_line_graphics_data()
         refs = layers(s)
         self.ae(len(refs), 0)
@@ -786,7 +842,7 @@ class TestGraphics(BaseTest):
         s.draw("\U0010EEEE\u0305\u0305\u059C\U0010EEEE\u0305\u030D\u059C")
         # Check that we can continue by using implicit row/column specification.
         s.draw("\U0010EEEE\u0305\U0010EEEE")
-        s.cursor_back(6)
+        s.cursor_move(6)
         s.update_only_line_graphics_data()
         refs = layers(s)
         self.ae(len(refs), 2)
@@ -800,7 +856,7 @@ class TestGraphics(BaseTest):
         s.draw("\U0010EEEE\u0305\u0305\u0305\U0010EEEE")
         s.apply_sgr("38;5;43")
         s.draw("\U0010EEEE\u0305\u0305\u059C\U0010EEEE\U0010EEEE\u0305\U0010EEEE")
-        s.cursor_back(6)
+        s.cursor_move(6)
         s.update_only_line_graphics_data()
         refs = layers(s)
         self.ae(len(refs), 2)
@@ -996,9 +1052,26 @@ class TestGraphics(BaseTest):
                 cmd += ',' + ','.join(f'{k}={v}' for k, v in kw.items())
             send_command(s, cmd)
 
+        iid = put_image(s, cw, ch, a='t')[0]
+        self.ae(s.grman.image_count, 1)
+        delete('I', i=iid)
+        self.ae(s.grman.image_count, 0)
+        iid1 = put_image(s, cw, ch, a='t')[0]
+        iid2 = put_image(s, cw, ch, a='t')[0]
+        self.ae(s.grman.image_count, 2)
+        delete('R', x=iid1, y=iid2)
+        self.ae(s.grman.image_count, 0)
+
         put_image(s, cw, ch)
         delete()
-        self.ae(len(layers(s)), 0), self.ae(s.grman.image_count, 1)
+        self.ae(s.grman.image_count, 1)
+        self.ae(len(layers(s)), 0)
+        delete('A')
+        self.ae(s.grman.image_count, 1)
+        s.reset()
+        self.ae(s.grman.image_count, 0)
+        put_image(s, cw, ch)
+        self.ae(s.grman.image_count, 1)
         delete('A')
         self.ae(s.grman.image_count, 0)
         self.assertEqual(s.grman.disk_cache.total_size, 0)
@@ -1047,6 +1120,18 @@ class TestGraphics(BaseTest):
         self.ae(s.grman.image_count, 0)
         self.assertEqual(s.grman.disk_cache.total_size, 0)
 
+        # test delete but not free
+        s.reset()
+        iid = 9999999
+        self.ae(put_image(s, cw, ch, id=iid), (iid, 'OK'))
+        self.ae(put_ref(s, id=iid), (iid, ('OK', f'i={iid}')))
+        self.ae(put_image(s, cw, ch, id=iid+1), (iid+1, 'OK'))
+        self.ae(put_ref(s, id=iid+1), (iid+1, ('OK', f'i={iid+1}')))
+        delete('i', i=iid)
+        self.ae(s.grman.image_count, 2)
+        delete('I', i=iid+1)
+        self.ae(s.grman.image_count, 1)
+
     def test_animation_frame_loading(self):
         s = self.create_screen()
         g = s.grman
@@ -1153,6 +1238,7 @@ class TestGraphics(BaseTest):
         self.assertIsNone(li(a='d', d='f', i=1))
         img = g.image_for_client_id(1)
         self.assertEqual(img['data'], b'5' * 36)
+        self.ae(g.image_count, 1)
         self.assertIsNone(li(a='d', d='F', i=1))
         self.ae(g.image_count, 0)
         self.assertEqual(g.disk_cache.total_size, 0)
@@ -1204,3 +1290,40 @@ class TestGraphics(BaseTest):
         s.reset()
         self.ae(g.image_count, 0)
         self.assertEqual(g.disk_cache.total_size, 0)
+
+    @unittest.skipIf(Image is None, 'PIL not available, skipping PNG tests')
+    def test_cached_rgba_conversion(self):
+        from kitty.render_cache import ImageRenderCacheForTesting
+        w, h = 5, 3
+        rgba_data = byte_block(w * h * 4)
+        img = Image.frombytes('RGBA', (w, h), rgba_data)
+        buf = BytesIO()
+        img.save(buf, 'PNG')
+        png_data = buf.getvalue()
+        with tempfile.TemporaryDirectory() as cache_path:
+            irc = ImageRenderCacheForTesting(cache_path)
+            srcs, outputs = [], []
+            for i in range(2 * irc.max_entries):
+                with open(os.path.join(cache_path, f'{i}.png'), 'wb') as f:
+                    f.write(png_data)
+                srcs.append(f.name)
+                outputs.append(irc.render(f.name))
+                entries = list(irc.entries())
+                self.assertLessEqual(len(entries), irc.max_entries)
+            self.ae(irc.num_of_renders, len(outputs))
+            remaining_outputs = outputs[-irc.max_entries:]
+            for x in remaining_outputs:
+                self.assertTrue(os.path.exists(x))
+            for x in outputs[:-irc.max_entries]:
+                self.assertFalse(os.path.exists(x))
+            self.assertLess(os.path.getmtime(remaining_outputs[0]), os.path.getmtime(remaining_outputs[1]))
+            remaining_srcs = srcs[-irc.max_entries:]
+            self.ae(irc.render(remaining_srcs[0]), remaining_outputs[0])
+            self.ae(irc.num_of_renders, len(outputs))
+            self.assertGreater(os.path.getmtime(remaining_outputs[0]), os.path.getmtime(remaining_outputs[1]))
+
+            width, height, fd = irc(remaining_srcs[-1])
+            with open(fd, 'rb') as f:
+                self.ae((width, height), (w, h))
+                f.seek(8)
+                self.ae(rgba_data, f.read())
