@@ -234,6 +234,10 @@ def conf_parsing(self):
     self.ae(opts.mouse_hide_wait[3], True)
     self.ae(opts.color23, Color(255, 0, 0))
     self.assertFalse(opts.keyboard_modes[''].keymap)
+    opts = p("url_excluded_characters '''")
+    self.ae(opts.url_excluded_characters, "'''")
+    opts = p("url_excluded_characters abc'")
+    self.ae(opts.url_excluded_characters, "abc'")
     opts = p('clear_all_shortcuts y', 'map f1 next_window')
     self.ae(len(opts.keyboard_modes[''].keymap), 1)
     opts = p('clear_all_mouse_actions y', 'mouse_map left click ungrabbed mouse_click_url_or_select')
