@@ -36,8 +36,8 @@ ap = re.search(r"^appname: str\s+=\s+'([^']+)'", raw, flags=re.MULTILINE)
 if ap is not None:
     appname = ap.group(1)
 
-ALL_ACTIONS = 'local_build man html build tag sdist upload website'.split()
-NIGHTLY_ACTIONS = 'local_build man html build sdist upload_nightly'.split()
+ALL_ACTIONS = 'local_build man html build tag sdist sbom upload website'.split()
+NIGHTLY_ACTIONS = 'local_build man html build sdist sbom upload_nightly'.split()
 
 
 def echo_cmd(cmd: Iterable[str]) -> None:
@@ -45,18 +45,28 @@ def echo_cmd(cmd: Iterable[str]) -> None:
     end = '\n'
     if isatty:
         end = f'\x1b[m{end}'
-        print('\x1b[92m', end='')
+        print('\x1b[32m', end='')  # ]]]]]
     print(shlex.join(cmd), end=end, flush=True)
 
 
-def call(*cmd: str, cwd: Optional[str] = None, echo: bool = False) -> None:
+def call(*cmd: str, cwd: Optional[str] = None, echo: bool = False, timeout: float | None = None) -> None:
     if len(cmd) == 1:
         q = shlex.split(cmd[0])
     else:
         q = list(cmd)
     if echo:
         echo_cmd(cmd)
-    ret = subprocess.Popen(q, cwd=cwd).wait()
+    p = subprocess.Popen(q, cwd=cwd)
+    try:
+        ret = p.wait(timeout)
+    except subprocess.TimeoutExpired:
+        p.terminate()
+        try:
+            p.wait(1)
+        except subprocess.TimeoutExpired:
+            p.kill()
+            p.wait()
+        raise
     if ret != 0:
         raise SystemExit(ret)
 
@@ -71,23 +81,22 @@ def run_build(args: Any) -> None:
     m = runpy.run_path('./setup.py', run_name='__publish__')
     vcs_rev: str = m['get_vcs_rev']()
 
-    def run_with_retry(cmd: str) -> None:
+    def run_with_retry(cmd: str, timeout: float | None = 20 * 60 ) -> None:
         try:
-            call(cmd, echo=True)
+            call(cmd, echo=True, timeout=timeout)
         except (SystemExit, Exception):
-            needs_retry = 'arm64' in cmd or building_nightly
+            needs_retry = building_nightly and 'linux' not in cmd
             if not needs_retry:
                 raise
-            print('Build failed, retrying in a few seconds...', file=sys.stderr)
+            print('Build failed, retrying in a minute seconds...', file=sys.stderr)
             if 'macos' in cmd:
                 call('python ../bypy macos shutdown')
-            time.sleep(25)
-            call(cmd, echo=True)
+            time.sleep(60)
+            call(cmd, echo=True, timeout=timeout)
 
-    for x in ('64', '32', 'arm64'):
+    for x in ('64', 'arm64'):
         prefix = f'python ../bypy linux --arch {x} '
         run_with_retry(prefix + f'program --non-interactive --extra-program-data "{vcs_rev}"')
-        call(prefix + 'shutdown', echo=True)
     run_with_retry(f'python ../bypy macos program --sign-installers --notarize --non-interactive --extra-program-data "{vcs_rev}"')
     call('python ../bypy macos shutdown', echo=True)
     call('make debug')
@@ -198,6 +207,10 @@ def run_sdist(args: Any) -> None:
         sign_file(f'{dest}.xz')
 
 
+def run_sbom(args: Any) -> None:
+    call(f'python ../bypy sbom --output build/kitty-{version}.tar.xz.spdx.json --url https://sw.kovidgoyal.net/kitty/binary kovidgoyal/{appname} {version}')
+
+
 class ReadFileWithProgressReporting(io.FileIO):  # {{{
     def __init__(self, path: str):
         super().__init__(path, 'rb')
@@ -210,7 +223,7 @@ class ReadFileWithProgressReporting(io.FileIO):  # {{{
     def __len__(self) -> int:
         return self._total
 
-    def read(self, size: int = -1) -> bytes:
+    def read(self, size: Optional[int] = -1) -> bytes:
         data = io.FileIO.read(self, size)
         if data:
             self.report_progress(len(data))
@@ -333,7 +346,7 @@ class GitHub:  # {{{
 
     def update_nightly_description(self, release_id: int) -> None:
         url = f'{self.url_base}/{release_id}'
-        now = str(datetime.datetime.utcnow()).split('.')[0] + ' UTC'
+        now = str(datetime.datetime.now(datetime.timezone.utc)).split('.')[0] + ' UTC'
         commit = subprocess.check_output(['git', 'rev-parse', '--verify', '--end-of-options', 'master^{commit}']).decode('utf-8').strip()
         self.patch(
             url, 'Failed to update nightly release description',
@@ -442,7 +455,6 @@ def files_for_upload() -> Dict[str, str]:
     for f, desc in {
         'macos/dist/kitty-{}.dmg': 'macOS dmg',
         'linux/64/dist/kitty-{}-x86_64.txz': 'Linux amd64 binary bundle',
-        'linux/32/dist/kitty-{}-i686.txz': 'Linux x86 binary bundle',
         'linux/arm64/dist/kitty-{}-arm64.txz': 'Linux arm64 binary bundle',
     }.items():
         path = os.path.join('bypy', 'b', f.format(version))
@@ -463,6 +475,7 @@ def files_for_upload() -> Dict[str, str]:
 
     files[f'build/kitty-{version}.tar.xz'] = 'Source code'
     files[f'build/kitty-{version}.tar.xz.sig'] = 'Source code GPG signature'
+    files[f'build/kitty-{version}.tar.xz.spdx.json'] = 'SBOM for kitty (all builds)'
     for path, desc in signatures.items():
         sign_file(path)
         files[f'{path}.sig'] = desc
@@ -504,6 +517,15 @@ def safe_read(path: str) -> str:
     return ''
 
 
+def remove_pycache_only_folders() -> None:
+    folders_to_remove = []
+    for dirpath, folders, files in os.walk('.'):
+        if not files and folders == ['__pycache__']:
+            folders_to_remove.append(dirpath)
+    for x in folders_to_remove:
+        shutil.rmtree(x)
+
+
 @contextmanager
 def change_to_git_master() -> Generator[None, None, None]:
     stash_ref_before = safe_read('.git/refs/stash')
@@ -512,13 +534,14 @@ def change_to_git_master() -> Generator[None, None, None]:
         branch_before = current_branch()
         if branch_before != 'master':
             subprocess.check_call(['git', 'switch', 'master'])
-            subprocess.check_call(['make', 'debug'])
+            remove_pycache_only_folders()
+            subprocess.check_call(['make', 'clean', 'debug'])
         try:
             yield
         finally:
             if branch_before != 'master':
                 subprocess.check_call(['git', 'switch', branch_before])
-                subprocess.check_call(['make', 'debug'])
+                subprocess.check_call(['make', 'clean', 'debug'])
     finally:
         if stash_ref_before != safe_read('.git/refs/stash'):
             subprocess.check_call(['git', 'stash', 'pop'])
@@ -562,7 +585,7 @@ def main() -> None:
         with change_to_git_master():
             building_nightly = True
             exec_actions(NIGHTLY_ACTIONS, args)
-            subprocess.run(['make', 'debug'])
+            subprocess.run(['make', 'clean', 'debug'])
         return
     require_git_master()
     if args.action == 'all':
