@@ -3,13 +3,13 @@
 package at
 
 import (
-	"kitty/tools/utils"
+	"strings"
 )
 
 func parse_key_val_args(args []string) map[escaped_string]escaped_string {
 	ans := make(map[escaped_string]escaped_string, len(args))
 	for _, arg := range args {
-		key, value, found := utils.Cut(arg, "=")
+		key, value, found := strings.Cut(arg, "=")
 		if found {
 			ans[escaped_string(key)] = escaped_string(value)
 		} else {
