@@ -11,12 +11,12 @@ import (
 	"strings"
 	"time"
 
-	"kitty/tools/cli"
-	"kitty/tools/cli/markup"
-	"kitty/tools/tui/loop"
-	"kitty/tools/tui/readline"
-	"kitty/tools/utils"
-	"kitty/tools/utils/shlex"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/cli/markup"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/tui/readline"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/shlex"
 )
 
 var _ = fmt.Print
@@ -203,6 +203,7 @@ func shell_main(cmd *cli.Command, args []string) (int, error) {
 	if err != nil {
 		return 1, err
 	}
+	running_shell = true
 	formatter = markup.New(true)
 	fmt.Println("Welcome to the kitty shell!")
 	fmt.Println("Use", formatter.Green("help"), "for assistance or", formatter.Green("exit"), "to quit.")
@@ -215,7 +216,17 @@ func shell_main(cmd *cli.Command, args []string) (int, error) {
 		}
 		fmt.Println(amsg)
 	}
-	rl := readline.New(nil, readline.RlInit{Prompt: prompt, Completer: completions, HistoryPath: filepath.Join(utils.CacheDir(), "shell.history.json")})
+	var rl *readline.Readline
+	combined_completer := func(before_cursor, after_cursor string) *cli.Completions {
+		ans := completions(before_cursor, after_cursor)
+		history := rl.HistoryCompleter(before_cursor, after_cursor)
+		for _, group := range history.Groups {
+			ans.MergeMatchGroup(group)
+		}
+		return ans
+	}
+
+	rl = readline.New(nil, readline.RlInit{Prompt: prompt, Completer: combined_completer, HistoryPath: filepath.Join(utils.CacheDir(), "shell.history.json")})
 	defer func() {
 		rl.Shutdown()
 	}()
