@@ -4,15 +4,15 @@ package update_self
 
 import (
 	"fmt"
-	"kitty"
 	"os"
 	"path/filepath"
 	"runtime"
 
-	"kitty/tools/cli"
-	"kitty/tools/tty"
-	"kitty/tools/tui"
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui"
+	"github.com/kovidgoyal/kitty/tools/utils"
 
 	"golang.org/x/sys/unix"
 )
@@ -33,7 +33,7 @@ func update_self(version string) (err error) {
 	if err != nil {
 		return err
 	}
-	if !kitty.IsStandaloneBuild {
+	if kitty.IsStandaloneBuild == "" {
 		return fmt.Errorf("This is not a standalone kitten executable. You must update all of kitty instead.")
 	}
 	rv := "v" + version
