@@ -4,14 +4,14 @@ package config
 
 import (
 	"fmt"
-	"kitty/tools/tui/loop"
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
 	"regexp"
+	"slices"
 	"strconv"
 	"strings"
-
-	"golang.org/x/exp/maps"
-	"golang.org/x/exp/slices"
+	"sync"
+	"unicode/utf8"
 )
 
 var _ = fmt.Print
@@ -30,7 +30,7 @@ func ParseStrDict(val, record_sep, field_sep string) (map[string]string, error)
 func PositiveFloat(val string) (ans float64, err error) {
 	ans, err = strconv.ParseFloat(val, 64)
 	if err == nil {
-		ans = utils.Max(0, ans)
+		ans = max(0, ans)
 	}
 	return
 }
@@ -38,7 +38,7 @@ func PositiveFloat(val string) (ans float64, err error) {
 func UnitFloat(val string) (ans float64, err error) {
 	ans, err = strconv.ParseFloat(val, 64)
 	if err == nil {
-		ans = utils.Max(0, utils.Min(ans, 1))
+		ans = max(0, min(ans, 1))
 	}
 	return
 }
@@ -60,8 +60,9 @@ func StringLiteral(val string) (string, error) {
 	var state State
 	decode := func(base int) {
 		text := string(buf[:bufcount])
-		num, _ := strconv.ParseUint(text, base, 32)
-		ans.WriteRune(rune(num))
+		if num, err := strconv.ParseUint(text, base, 32); err == nil && num <= utf8.MaxRune {
+			ans.WriteRune(rune(num))
+		}
 		state = normal
 		bufcount = 0
 		buflimit = 0
@@ -189,7 +190,7 @@ func StringLiteral(val string) (string, error) {
 	return ans.String(), nil
 }
 
-var ModMap = utils.Once(func() map[string]string {
+var ModMap = sync.OnceValue(func() map[string]string {
 	return map[string]string{
 		"shift":     "shift",
 		"⇧":         "shift",
@@ -211,7 +212,7 @@ var ModMap = utils.Once(func() map[string]string {
 	}
 })
 
-var ShortcutSpecPat = utils.Once(func() *regexp.Regexp {
+var ShortcutSpecPat = sync.OnceValue(func() *regexp.Regexp {
 	return regexp.MustCompile(`([^+])>`)
 })
 
@@ -308,5 +309,5 @@ func ResolveShortcuts(actions []*KeyAction) []*KeyAction {
 			action_map[key] = ac
 		}
 	}
-	return maps.Values(action_map)
+	return utils.Values(action_map)
 }
