@@ -5,9 +5,12 @@ import (
 	"encoding/hex"
 	"encoding/json"
 	"fmt"
+	"io/fs"
+	"maps"
 	"os"
 	"path/filepath"
 	"slices"
+	"syscall"
 	"time"
 
 	"github.com/kovidgoyal/kitty/tools/utils"
@@ -15,6 +18,48 @@ import (
 
 var _ = fmt.Print
 
+type file_state struct {
+	Size    int64
+	ModTime time.Time
+	Inode   uint64
+}
+
+func (s file_state) String() string {
+	return fmt.Sprintf("fs{Size: %d, Inode: %d, ModTime: %s}", s.Size, s.Inode, s.ModTime)
+}
+
+func (s *file_state) equal(o *file_state) bool {
+	return o != nil && s.Size == o.Size && s.ModTime.Equal(o.ModTime) && s.Inode == o.Inode
+}
+
+func get_file_state(fi fs.FileInfo) *file_state {
+	// The Sys() method returns the underlying data source (can be nil).
+	// For Unix-like systems, it's a *syscall.Stat_t.
+	stat, ok := fi.Sys().(*syscall.Stat_t)
+	if !ok {
+		// For non-Unix systems, you might not have an inode.
+		// In that case, you can fall back to using only size and mod time.
+		return &file_state{
+			Size:    fi.Size(),
+			ModTime: fi.ModTime(),
+			Inode:   0, // Inode not available
+		}
+	}
+	return &file_state{
+		Size:    fi.Size(),
+		ModTime: fi.ModTime(),
+		Inode:   stat.Ino,
+	}
+}
+
+func get_file_state_from_path(path string) (*file_state, error) {
+	if s, err := os.Stat(path); err != nil {
+		return nil, err
+	} else {
+		return get_file_state(s), nil
+	}
+}
+
 func new_disk_cache(path string, max_size int64) (dc *DiskCache, err error) {
 	if path, err = filepath.Abs(path); err != nil {
 		return
@@ -28,12 +73,13 @@ func new_disk_cache(path string, max_size int64) (dc *DiskCache, err error) {
 	if err = ans.ensure_entries(); err != nil {
 		return
 	}
-	if pruned, err := ans.prune(); err != nil {
-		return nil, err
-	} else if pruned {
-		if err = ans.write_entries(); err != nil {
-			return nil, err
+	defer func() {
+		if we := ans.write_entries_if_dirty(); we != nil && err == nil {
+			err = we
 		}
+	}()
+	if _, err := ans.prune(); err != nil {
+		return nil, err
 	}
 	if ans.get_dir, err = os.MkdirTemp(ans.Path, "getdir-*"); err != nil {
 		return
@@ -85,19 +131,50 @@ func (dc *DiskCache) unlock() {
 
 func (dc *DiskCache) entries_path() string { return filepath.Join(dc.Path, "entries.json") }
 
-func (dc *DiskCache) write_entries() (err error) {
+func (dc *DiskCache) write_entries_if_dirty() (err error) {
+	if !dc.entries_dirty {
+		return
+	}
+	path := dc.entries_path()
+	defer func() {
+		if err == nil {
+			dc.entries_dirty = false
+			if s, serr := get_file_state_from_path(path); serr == nil {
+				dc.entries_last_read_state = s
+			}
+		}
+	}()
 	if d, err := json.Marshal(dc.entries); err != nil {
 		return err
 	} else {
-		return os.WriteFile(dc.entries_path(), d, 0o600)
+		// use a rename so that the inode number changes
+		// dont bother with full utils.AtomicWriteFile() as it is slower
+		temp := path + ".temp"
+		removed := false
+		defer func() {
+			if !removed {
+				_ = os.Remove(temp)
+				removed = true
+			}
+		}()
+		if err = os.WriteFile(temp, d, 0o600); err == nil {
+			if err = os.Rename(temp, path); err == nil {
+				removed = true
+			}
+		}
+		return err
 	}
 }
 
-func (dc *DiskCache) rebuild_entries() error {
+func (e Entry) String() string {
+	return fmt.Sprintf("Entry{Key: %s, Size: %d, LastUsed: %s}", e.Key, e.Size, e.LastUsed)
+}
+
+func (dc *DiskCache) entries_from_folders() (total_size int64, ans map[string]*Entry, sorted []*Entry, err error) {
 	if entries, err := os.ReadDir(dc.Path); err != nil {
-		return err
+		return 0, nil, nil, err
 	} else {
-		ans := make(map[string]*Entry)
+		ans = make(map[string]*Entry)
 		var total int64
 		for _, x := range entries {
 			if x.IsDir() {
@@ -105,7 +182,7 @@ func (dc *DiskCache) rebuild_entries() error {
 					key := sub_entries[0].Name()
 					path := dc.folder_for_key(key)
 					if file_entries, err := os.ReadDir(path); err == nil {
-						e := Entry{}
+						e := Entry{Key: key}
 						for _, f := range file_entries {
 							if fi, err := f.Info(); err == nil {
 								e.Size += fi.Size()
@@ -120,37 +197,58 @@ func (dc *DiskCache) rebuild_entries() error {
 				}
 			}
 		}
-		sorted := utils.Values(ans)
+		sorted = utils.Values(ans)
 		slices.SortFunc(sorted, func(a, b *Entry) int {
 			return a.LastUsed.Compare(b.LastUsed)
 		})
-		dc.entries = Metadata{TotalSize: total, SortedEntries: sorted}
-		dc.entry_map = ans
+		return total, ans, sorted, nil
 	}
-	return nil
+}
+
+func (dc *DiskCache) rebuild_entries() error {
+	total, ans, sorted, err := dc.entries_from_folders()
+	if err != nil {
+		return err
+	}
+	dc.entries = Metadata{TotalSize: total, SortedEntries: sorted, PathMap: make(map[string]string)}
+	dc.entry_map = ans
+	dc.entries_dirty = true
+	return dc.write_entries_if_dirty()
 }
 
 func (dc *DiskCache) ensure_entries() error {
-	needed := dc.entry_map == nil
+	needed := dc.entry_map == nil || dc.entries_last_read_state == nil
 	path := dc.entries_path()
+	var fstate *file_state
 	if !needed {
-		if s, err := os.Stat(path); err == nil && s.ModTime().After(dc.entries_mod_time) {
-			needed = true
+		if s, err := get_file_state_from_path(path); err == nil {
+			fstate = s
+			if !s.equal(dc.entries_last_read_state) {
+				needed = true
+			}
 		}
 	}
 	if needed {
 		if data, err := os.ReadFile(path); err != nil {
 			if os.IsNotExist(err) {
 				dc.entry_map = make(map[string]*Entry)
-				dc.entries = Metadata{SortedEntries: make([]*Entry, 0)}
+				dc.entries = Metadata{SortedEntries: make([]*Entry, 0), PathMap: make(map[string]string)}
 			} else {
 				return err
 			}
 		} else {
-			dc.entries = Metadata{SortedEntries: make([]*Entry, 0)}
+			dc.read_count += 1
+			dc.entries = Metadata{SortedEntries: make([]*Entry, 0), PathMap: make(map[string]string)}
 			if err := json.Unmarshal(data, &dc.entries); err != nil {
 				// corrupted data
 				dc.rebuild_entries()
+			} else {
+				if fstate == nil {
+					if s, err := get_file_state_from_path(path); err == nil {
+						fstate = s
+					}
+				}
+				dc.entries_last_read_state = fstate
 			}
 			dc.entry_map = make(map[string]*Entry)
 			for _, e := range dc.entries.SortedEntries {
@@ -170,43 +268,68 @@ func (dc *DiskCache) folder_for_key(key string) (ans string) {
 	return filepath.Join(dc.Path, ans)
 }
 
-func (dc *DiskCache) update_last_used(key string) {
-	if dc.ensure_entries() == nil {
-		dc.update_timestamp(key)
+func (dc *DiskCache) export_to_get_dir(key, path string) (string, error) {
+	dest := filepath.Join(dc.get_dir, key+"-"+filepath.Base(path))
+	if err := os.Link(path, dest); err != nil {
+		os.Remove(dest)
+		if err := os.Link(path, dest); err != nil {
+			return "", err
+		}
 	}
+	return dest, nil
 
 }
 
-func (dc *DiskCache) get(key string, items []string) map[string]string {
-	ans := make(map[string]string, len(items))
-	base := dc.folder_for_key(key)
-	if s, err := os.Stat(base); err != nil || !s.IsDir() {
-		return ans
+func (dc *DiskCache) get(key string, items []string) (map[string]string, error) {
+	if err := dc.ensure_entries(); err != nil {
+		return nil, err
 	}
+	base := dc.folder_for_key(key)
+	if len(items) == 0 {
+		if entries, err := os.ReadDir(base); err != nil {
+			if os.IsNotExist(err) {
+				err = nil
+			}
+			return nil, err
+		} else {
+			for _, e := range entries {
+				items = append(items, e.Name())
+			}
+		}
+	} else {
+		if s, err := os.Stat(base); err != nil || !s.IsDir() {
+			if os.IsNotExist(err) {
+				err = nil
+			}
+			return nil, err
+		}
+	}
+	ans := make(map[string]string, len(items))
 	for _, x := range items {
 		p := filepath.Join(base, x)
 		if s, err := os.Stat(p); err != nil || s.IsDir() {
 			continue
 		}
-		dest := filepath.Join(dc.get_dir, key+"-"+x)
-		if err := os.Link(p, dest); err != nil {
-			os.Remove(dest)
-			if err := os.Link(p, dest); err != nil {
-				dest = ""
-			}
-		}
+		dest, _ := dc.export_to_get_dir(key, p)
 		if dest != "" {
 			ans[x] = dest
 		}
 	}
-	dc.update_last_used(key)
-	return ans
+	if len(items) > 0 {
+		dc.update_timestamp(key)
+	}
+	return ans, dc.write_entries_if_dirty()
 }
 
 func (dc *DiskCache) remove(key string) (err error) {
 	if err = dc.ensure_entries(); err != nil {
 		return
 	}
+	defer func() {
+		if we := dc.write_entries_if_dirty(); we != nil && err == nil {
+			err = we
+		}
+	}()
 	base := dc.folder_for_key(key)
 	if err = os.RemoveAll(base); err == nil {
 		t := dc.entry_map[key]
@@ -214,7 +337,7 @@ func (dc *DiskCache) remove(key string) (err error) {
 			delete(dc.entry_map, key)
 			dc.entries.TotalSize = max(0, dc.entries.TotalSize-t.Size)
 			dc.entries.SortedEntries = utils.Filter(dc.entries.SortedEntries, func(x *Entry) bool { return x.Key != key })
-			return dc.write_entries()
+			dc.entries_dirty = true
 		}
 	}
 	return
@@ -229,8 +352,10 @@ func (dc *DiskCache) prune() (bool, error) {
 		if err := os.RemoveAll(base); err == nil {
 			t := dc.entries.SortedEntries[0]
 			delete(dc.entry_map, t.Key)
+			maps.DeleteFunc(dc.entries.PathMap, func(path, key string) bool { return key == t.Key })
 			dc.entries.TotalSize = max(0, dc.entries.TotalSize-t.Size)
 			dc.entries.SortedEntries = dc.entries.SortedEntries[1:]
+			dc.entries_dirty = true
 		} else {
 			return false, err
 		}
@@ -244,6 +369,7 @@ func (dc *DiskCache) update_timestamp(key string) {
 	idx := slices.Index(dc.entries.SortedEntries, t)
 	copy(dc.entries.SortedEntries[idx:], dc.entries.SortedEntries[idx+1:])
 	dc.entries.SortedEntries[len(dc.entries.SortedEntries)-1] = t
+	dc.entries_dirty = true
 }
 
 func (dc *DiskCache) update_accounting(key string, changed int64) (err error) {
@@ -257,25 +383,52 @@ func (dc *DiskCache) update_accounting(key string, changed int64) (err error) {
 	t.Size += changed
 	t.Size = max(0, t.Size)
 	dc.entries.TotalSize += t.Size - old_size
+	dc.entries_dirty = true
 	dc.update_timestamp(key)
 	dc.prune()
-	return dc.write_entries()
+	return dc.write_entries_if_dirty()
 }
 
 func (dc *DiskCache) keys() (ans []string, err error) {
 	if err = dc.ensure_entries(); err != nil {
 		return
 	}
-	return utils.Keys(dc.entry_map), nil
+	ans = make([]string, len(dc.entries.SortedEntries))
+	for i, e := range dc.entries.SortedEntries {
+		ans[i] = e.Key
+	}
+	return
 }
 
-func (dc *DiskCache) add(key string, items map[string][]byte) (err error) {
+func (dc *DiskCache) add_path(path, key string, items map[string][]byte) (ans map[string]string, err error) {
+	if err = dc.ensure_entries(); err != nil {
+		return
+	}
+	defer func() {
+		if we := dc.write_entries_if_dirty(); we != nil && err == nil {
+			err = we
+		}
+	}()
+
+	if existing := dc.entries.PathMap[path]; existing != "" && existing != key {
+		delete(dc.entries.PathMap, path)
+		dc.entries_dirty = true
+		if err = dc.remove(existing); err != nil {
+			return
+		}
+	}
+	dc.entries.PathMap[path] = key
+	dc.entries_dirty = true
+	return dc.add(key, items)
+}
+
+func (dc *DiskCache) add(key string, items map[string][]byte) (ans map[string]string, err error) {
 	if err = dc.ensure_entries(); err != nil {
 		return
 	}
 	base := dc.folder_for_key(key)
 	if err = os.MkdirAll(base, 0o700); err != nil {
-		return err
+		return
 	}
 	var changed int64
 	defer func() {
@@ -284,22 +437,40 @@ func (dc *DiskCache) add(key string, items map[string][]byte) (err error) {
 			err = e
 		}
 	}()
+	ans = make(map[string]string, len(items))
 	for x, data := range items {
 		p := filepath.Join(base, x)
 		var before int64
+		exists := false
 		if s, err := os.Stat(p); err == nil {
 			before = s.Size()
+			exists = true
 		}
 		if len(data) == 0 {
-			if err = os.Remove(p); err != nil {
-				return
+			if exists {
+				if err = os.Remove(p); err != nil {
+					if !os.IsNotExist(err) {
+						return
+					}
+					err = nil
+				}
+				changed -= before
 			}
-			changed -= before
 		} else {
+			// unlink the file so that writing to it does not change a
+			// previously linked copy created by get()
+			if exists {
+				_ = os.Remove(p)
+			}
 			if err = os.WriteFile(p, data, 0o700); err != nil {
 				return
 			}
 			changed += int64(len(data)) - before
+			if dest, err := dc.export_to_get_dir(key, p); err != nil {
+				return ans, err
+			} else {
+				ans[x] = dest
+			}
 		}
 	}
 	return
