@@ -806,7 +806,7 @@ func (f *Function) SetRegisterTo(self Register, val any) {
 }
 
 func (r Register) ARMId() uint32 {
-	num, err := strconv.Atoi(r.Name[1:])
+	num, err := strconv.ParseUint(r.Name[1:], 10, 32)
 	if err != nil {
 		panic(err)
 	}
@@ -1151,7 +1151,7 @@ func (s *Function) OutputASM(w io.Writer) {
 	}
 	fmt.Fprint(w, "// ")
 	s.print_signature(w)
-	fmt.Fprintf(w, "\nTEXT ·%s(SB), NOSPLIT, $0-%d\n", s.Name, s.Size)
+	fmt.Fprintf(w, "\nTEXT ·%s(SB), NOSPLIT|TOPFRAME|NOFRAME, $0-%d\n", s.Name, s.Size)
 
 	has_trailing_return := false
 	for _, i := range s.Instructions {
