@@ -14,7 +14,7 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 const (
@@ -78,6 +78,10 @@ func SetReadTimeout(d time.Duration) TermiosOperation {
 
 var SetBlockingRead TermiosOperation = SetReadTimeout(0)
 
+var SetNoCanonical TermiosOperation = func(t *unix.Termios) {
+	t.Lflag &^= unix.ICANON
+}
+
 var SetRaw TermiosOperation = func(t *unix.Termios) {
 	// This attempts to replicate the behaviour documented for cfmakeraw in
 	// the termios(3) manpage, as Go doesn't wrap cfmakeraw probably because its not in POSIX
@@ -179,6 +183,9 @@ func (self *Term) set_termios_attrs(when uintptr, modify func(*unix.Termios)) (e
 }
 
 func (self *Term) ApplyOperations(when uintptr, operations ...TermiosOperation) (err error) {
+	if len(operations) == 0 {
+		return
+	}
 	return self.set_termios_attrs(when, func(t *unix.Termios) {
 		for _, op := range operations {
 			op(t)
@@ -214,7 +221,7 @@ func (self *Term) Restore() error {
 }
 
 func (self *Term) RestoreAndClose() error {
-	self.Restore()
+	_ = self.Restore()
 	return self.Close()
 }
 
@@ -240,7 +247,9 @@ func (self *Term) SuspendAndRun(callback func() error) error {
 		return err
 	}
 	err = callback()
-	resume()
+	if rerr := resume(); rerr != nil {
+		err = rerr
+	}
 	return err
 }
 
@@ -280,8 +289,19 @@ func (self *Term) ReadWithTimeout(b []byte, d time.Duration) (n int, err error)
 	}
 }
 
-func (self *Term) Read(b []byte) (int, error) {
-	return self.os_file.Read(b)
+func is_temporary_read_error(err error) bool {
+	return errors.Is(err, unix.EINTR) || errors.Is(err, unix.EAGAIN) || errors.Is(err, unix.EWOULDBLOCK)
+}
+
+func (self *Term) Read(b []byte) (n int, err error) {
+	for {
+		n, err = self.os_file.Read(b)
+		// On macOS we get EAGAIN if another thread is writing to the tty at the same time
+		if err != nil && is_temporary_read_error(err) && n <= 0 {
+			continue
+		}
+		return
+	}
 }
 
 func (self *Term) Write(b []byte) (int, error) {
@@ -312,32 +332,33 @@ func (self *Term) WriteString(b string) (int, error) {
 }
 
 func (self *Term) DebugPrintln(a ...any) {
-	msg := []byte(fmt.Sprintln(a...))
+	msg := fmt.Appendln(nil, a...)
 	const limit = 2048
 	encoded := make([]byte, limit*2)
 	for i := 0; i < len(msg); i += limit {
-		end := i + limit
-		if end > len(msg) {
-			end = len(msg)
-		}
+		end := min(i+limit, len(msg))
 		chunk := msg[i:end]
 		encoded = encoded[:cap(encoded)]
 		base64.StdEncoding.Encode(encoded, chunk)
-		self.WriteString("\x1bP@kitty-print|")
-		self.Write(encoded)
-		self.WriteString("\x1b\\")
+		_, _ = self.WriteString("\x1bP@kitty-print|")
+		_, _ = self.Write(encoded)
+		_, _ = self.WriteString("\x1b\\")
 	}
 }
 
-func (self *Term) GetSize() (*unix.Winsize, error) {
+func GetSize(fd int) (*unix.Winsize, error) {
 	for {
-		sz, err := unix.IoctlGetWinsize(self.Fd(), unix.TIOCGWINSZ)
+		sz, err := unix.IoctlGetWinsize(fd, unix.TIOCGWINSZ)
 		if err != unix.EINTR {
 			return sz, err
 		}
 	}
 }
 
+func (self *Term) GetSize() (*unix.Winsize, error) {
+	return GetSize(self.Fd())
+}
+
 // go doesn't have a wrapper for ctermid()
 func Ctermid() string { return "/dev/tty" }
 
@@ -364,24 +385,20 @@ func DebugPrintln(a ...any) {
 	}
 }
 
-func DrainControllingTTY(wait_for time.Duration) {
-	tty, err := OpenControllingTerm(SetRaw, SetNoEcho)
+func ReadSingleByteFromTerminal() (b byte, err error) {
+	term, err := OpenControllingTerm(SetBlockingRead, SetNoCanonical)
 	if err != nil {
-		return
+		return 0, err
 	}
-	sel := utils.CreateSelect(1)
-	sel.RegisterRead(tty.Fd())
+	defer term.Close()
+	ans := []byte{b}
 	for {
-		n, err := sel.Wait(wait_for)
-		if err == unix.EINTR {
-			continue
-		}
+		n, err := term.Read(ans)
 		if err != nil {
-			return
+			return 0, err
 		}
-		if n > 0 && sel.IsReadyToRead(tty.Fd()) {
-			tty.Read(make([]byte, 256))
+		if n > 0 {
+			return ans[0], nil
 		}
-		break
 	}
 }
