@@ -14,6 +14,17 @@ const (
 	LIGHT_COLOR_PREFERENCE
 )
 
+func (c ColorPreference) String() string {
+	switch c {
+	case DARK_COLOR_PREFERENCE:
+		return "dark"
+	case LIGHT_COLOR_PREFERENCE:
+		return "light"
+	default:
+		return "no-preference"
+	}
+}
+
 type TerminalCapabilities struct {
 	KeyboardProtocol                 bool
 	KeyboardProtocolResponseReceived bool
