@@ -4,6 +4,7 @@ package tui
 
 import (
 	"fmt"
+	"strings"
 	"time"
 
 	"github.com/kovidgoyal/kitty"
@@ -205,10 +206,21 @@ func (ms *MouseSelection) DragScroll(ev *loop.MouseEvent, lp *loop.Loop, callbac
 	ms.drag_scroll.mouse_event = *ev
 }
 
+type Point struct {
+	X, Y int
+}
+
+func (p Point) Sub(other Point) Point {
+	return Point{X: p.X - other.X, Y: p.Y - other.Y}
+}
+
 type CellRegion struct {
-	TopLeft, BottomRight struct{ X, Y int }
+	TopLeft, BottomRight Point
 	Id                   string
-	OnClick              []func(id string) error
+	OnClick              []func(id string) error                                       // simple left click ignoring modifiers
+	OnClickEvent         func(id string, ev *loop.MouseEvent, cell_offset Point) error // any click event
+	PointerShape         loop.PointerShape
+	HoverStyle           string // set to "default" for the global hover style
 }
 
 func (c CellRegion) Contains(x, y int) bool { // 0-based
@@ -219,11 +231,12 @@ func (c CellRegion) Contains(x, y int) bool { // 0-based
 }
 
 type MouseState struct {
-	Cell, Pixel struct{ X, Y int }
+	Cell, Pixel Point
 	Pressed     struct{ Left, Right, Middle, Fourth, Fifth, Sixth, Seventh bool }
 
 	regions           []*CellRegion
 	region_id_map     map[string][]*CellRegion
+	region_line_map   map[int][]*CellRegion
 	hovered_ids       *utils.Set[string]
 	default_url_style struct {
 		value  string
@@ -231,25 +244,34 @@ type MouseState struct {
 	}
 }
 
-func (m *MouseState) AddCellRegion(id string, start_x, start_y, end_x, end_y int, on_click ...func(id string) error) *CellRegion {
-	cr := CellRegion{TopLeft: struct{ X, Y int }{start_x, start_y}, BottomRight: struct{ X, Y int }{end_x, end_y}, Id: id, OnClick: on_click}
+func (m *MouseState) add_region(cr CellRegion) *CellRegion {
 	m.regions = append(m.regions, &cr)
 	if m.region_id_map == nil {
 		m.region_id_map = make(map[string][]*CellRegion)
+		m.region_line_map = make(map[int][]*CellRegion)
+	}
+	m.region_id_map[cr.Id] = append(m.region_id_map[cr.Id], &cr)
+	for y := cr.TopLeft.Y; y <= cr.BottomRight.Y; y++ {
+		m.region_line_map[y] = append(m.region_line_map[y], &cr)
 	}
-	m.region_id_map[id] = append(m.region_id_map[id], &cr)
 	return &cr
 }
 
+func (m *MouseState) AddCellRegion(id string, start_x, start_y, end_x, end_y int, on_click ...func(id string) error) *CellRegion {
+	return m.add_region(CellRegion{
+		TopLeft: Point{start_x, start_y}, BottomRight: Point{end_x, end_y}, Id: id, OnClick: on_click, PointerShape: loop.POINTER_POINTER, HoverStyle: "default"})
+}
+
 func (m *MouseState) ClearCellRegions() {
 	m.regions = nil
 	m.region_id_map = nil
 	m.hovered_ids = nil
+	m.region_line_map = nil
 }
 
 func (m *MouseState) UpdateHoveredIds() (changed bool) {
 	h := utils.NewSet[string]()
-	for _, r := range m.regions {
+	for _, r := range m.region_line_map[m.Cell.Y] {
 		if r.Contains(m.Cell.X, m.Cell.Y) {
 			h.Add(r.Id)
 		}
@@ -260,7 +282,8 @@ func (m *MouseState) UpdateHoveredIds() (changed bool) {
 }
 
 func (m *MouseState) ApplyHoverStyles(lp *loop.Loop, style ...string) {
-	if m.hovered_ids == nil {
+	if m.hovered_ids == nil || m.hovered_ids.Len() == 0 {
+		lp.ClearPointerShapes()
 		return
 	}
 	hs := ""
@@ -287,21 +310,55 @@ func (m *MouseState) ApplyHoverStyles(lp *loop.Loop, style ...string) {
 		hs = style[0]
 	}
 	is_hovered := false
+	ps := loop.DEFAULT_POINTER
 	for id := range m.hovered_ids.Iterable() {
 		for _, r := range m.region_id_map[id] {
-			lp.StyleRegion(hs, r.TopLeft.X, r.TopLeft.Y, r.BottomRight.X, r.BottomRight.Y)
+			if r.HoverStyle != "" {
+				s := strings.Replace(r.HoverStyle, "default", hs, 1)
+				lp.StyleRegion(s, r.TopLeft.X, r.TopLeft.Y, r.BottomRight.X, r.BottomRight.Y)
+			}
 			is_hovered = true
+			ps = r.PointerShape
 		}
 	}
 	if is_hovered {
-		if s, has := lp.CurrentPointerShape(); !has || s != loop.POINTER_POINTER {
-			lp.PushPointerShape(loop.POINTER_POINTER)
+		if s, has := lp.CurrentPointerShape(); !has || s != ps {
+			lp.PushPointerShape(ps)
 		}
 	} else {
 		lp.ClearPointerShapes()
 	}
 }
 
+func (m *MouseState) DispatchEventToHoveredRegions(ev *loop.MouseEvent) error {
+	if ev.Event_type != loop.MOUSE_CLICK {
+		return nil
+	}
+	is_simple_click := ev.Buttons&loop.LEFT_MOUSE_BUTTON != 0
+	seen := utils.NewSet[string]()
+	for id := range m.hovered_ids.Iterable() {
+		for _, r := range m.region_id_map[id] {
+			if seen.Has(r.Id) {
+				continue
+			}
+			seen.Add(r.Id)
+			if is_simple_click {
+				for _, f := range r.OnClick {
+					if err := f(r.Id); err != nil {
+						return err
+					}
+				}
+			}
+			if r.OnClickEvent != nil {
+				if err := r.OnClickEvent(r.Id, ev, m.Cell.Sub(r.TopLeft)); err != nil {
+					return err
+				}
+			}
+		}
+	}
+	return nil
+}
+
 func (m *MouseState) ClickHoveredRegions() error {
 	seen := utils.NewSet[string]()
 	for id := range m.hovered_ids.Iterable() {
