@@ -6,23 +6,28 @@ import (
 	"archive/tar"
 	"bytes"
 	"fmt"
+	"maps"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"slices"
 	"strings"
 
-	"golang.org/x/exp/maps"
-	"golang.org/x/exp/slices"
-
-	"kitty"
-	"kitty/tools/tty"
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 var _ = fmt.Print
 
 type integration_setup_func = func(shell_integration_dir string, argv []string, env map[string]string) ([]string, map[string]string, error)
 
+func TerminfoData() string {
+	d := Data()
+	entry := d["terminfo/x/xterm-kitty"]
+	return utils.UnsafeBytesToString(entry.Data)
+}
+
 func extract_files(match, dest_dir string) (err error) {
 	d := Data()
 	for _, fname := range d.FilesMatching(match) {
@@ -45,7 +50,7 @@ func extract_files(match, dest_dir string) (err error) {
 			if existing, rerr := os.ReadFile(dest); rerr == nil && bytes.Equal(existing, entry.Data) {
 				continue
 			}
-			if err = utils.AtomicWriteFile(dest, entry.Data, 0o644); err != nil {
+			if err = utils.AtomicWriteFile(dest, bytes.NewReader(entry.Data), 0o644); err != nil {
 				return
 			}
 		}
