@@ -10,6 +10,7 @@ import (
 	"image"
 	"image/color"
 	"image/gif"
+	"image/png"
 	"io"
 	"os"
 	"os/exec"
@@ -53,6 +54,27 @@ type ImageFrame struct {
 	Img                      image.Image
 }
 
+type SerializableImageFrame struct {
+	Width, Height, Left, Top int
+	Number                   int // 1-based number
+	Compose_onto             int // number of frame to compose onto
+	Delay_ms                 int // negative for gapless frame, zero ignored, positive is number of ms
+	Is_opaque                bool
+	Size                     int
+}
+
+func (s SerializableImageFrame) NeededSize() int {
+	return utils.IfElse(s.Is_opaque, 3, 4) * s.Width * s.Height
+}
+
+func (s *ImageFrame) Serialize() SerializableImageFrame {
+	return SerializableImageFrame{
+		Width: s.Width, Height: s.Height, Left: s.Left, Top: s.Top,
+		Number: s.Number, Compose_onto: s.Compose_onto, Delay_ms: int(s.Delay_ms),
+		Is_opaque: s.Is_opaque,
+	}
+}
+
 func (self *ImageFrame) DataAsSHM(pattern string) (ans shm.MMap, err error) {
 	bytes_per_pixel := 4
 	if self.Is_opaque {
@@ -63,7 +85,7 @@ func (self *ImageFrame) DataAsSHM(pattern string) (ans shm.MMap, err error) {
 		return nil, err
 	}
 	switch img := self.Img.(type) {
-	case *NRGB:
+	case *imaging.NRGB:
 		if bytes_per_pixel == 3 {
 			copy(ans.Slice(), img.Pix)
 			return
@@ -78,7 +100,7 @@ func (self *ImageFrame) DataAsSHM(pattern string) (ans shm.MMap, err error) {
 	var final_img image.Image
 	switch bytes_per_pixel {
 	case 3:
-		rgb := &NRGB{Pix: ans.Slice(), Stride: bytes_per_pixel * self.Width, Rect: dest_rect}
+		rgb := &imaging.NRGB{Pix: ans.Slice(), Stride: bytes_per_pixel * self.Width, Rect: dest_rect}
 		final_img = rgb
 	case 4:
 		rgba := &image.NRGBA{Pix: ans.Slice(), Stride: bytes_per_pixel * self.Width, Rect: dest_rect}
@@ -96,7 +118,7 @@ func (self *ImageFrame) Data() (ans []byte) {
 		bytes_per_pixel = 3
 	}
 	switch img := self.Img.(type) {
-	case *NRGB:
+	case *imaging.NRGB:
 		if bytes_per_pixel == 3 {
 			return img.Pix
 		}
@@ -109,7 +131,7 @@ func (self *ImageFrame) Data() (ans []byte) {
 	var final_img image.Image
 	switch bytes_per_pixel {
 	case 3:
-		rgb := NewNRGB(dest_rect)
+		rgb := imaging.NewNRGB(dest_rect)
 		final_img = rgb
 		ans = rgb.Pix
 	case 4:
@@ -122,12 +144,82 @@ func (self *ImageFrame) Data() (ans []byte) {
 	return
 }
 
+func ImageFrameFromSerialized(s SerializableImageFrame, data []byte) (aa *ImageFrame, err error) {
+	ans := ImageFrame{
+		Width: s.Width, Height: s.Height, Left: s.Left, Top: s.Top,
+		Number: s.Number, Compose_onto: s.Compose_onto, Delay_ms: int32(s.Delay_ms),
+		Is_opaque: s.Is_opaque,
+	}
+	bytes_per_pixel := utils.IfElse(s.Is_opaque, 3, 4)
+	if expected := bytes_per_pixel * s.Width * s.Height; len(data) != expected {
+		return nil, fmt.Errorf("serialized image data has size: %d != %d", len(data), expected)
+	}
+	if s.Is_opaque {
+		ans.Img, err = imaging.NewNRGBWithContiguousRGBPixels(data, s.Left, s.Top, s.Width, s.Height)
+	} else {
+		ans.Img, err = NewNRGBAWithContiguousRGBAPixels(data, s.Left, s.Top, s.Width, s.Height)
+	}
+	return &ans, err
+}
+
 type ImageData struct {
 	Width, Height    int
 	Format_uppercase string
 	Frames           []*ImageFrame
 }
 
+type SerializableImageMetadata struct {
+	Version          int
+	Width, Height    int
+	Format_uppercase string
+	Frames           []SerializableImageFrame
+}
+
+const SERIALIZE_VERSION = 1
+
+func (self *ImageFrame) SaveAsUncompressedPNG(output io.Writer) error {
+	encoder := png.Encoder{CompressionLevel: png.NoCompression}
+	return encoder.Encode(output, self.Img)
+}
+
+func (self *ImageData) SerializeOnlyMetadata() SerializableImageMetadata {
+	f := make([]SerializableImageFrame, len(self.Frames))
+	for i, s := range self.Frames {
+		f[i] = s.Serialize()
+	}
+	return SerializableImageMetadata{Version: SERIALIZE_VERSION, Width: self.Width, Height: self.Height, Format_uppercase: self.Format_uppercase, Frames: f}
+}
+
+func (self *ImageData) Serialize() (SerializableImageMetadata, [][]byte) {
+	m := self.SerializeOnlyMetadata()
+	data := make([][]byte, len(self.Frames))
+	for i, f := range self.Frames {
+		data[i] = f.Data()
+		m.Frames[i].Size = len(data[i])
+	}
+	return m, data
+}
+
+func ImageFromSerialized(m SerializableImageMetadata, data [][]byte) (*ImageData, error) {
+	if m.Version > SERIALIZE_VERSION {
+		return nil, fmt.Errorf("serialized image data has unsupported version: %d", m.Version)
+	}
+	if len(m.Frames) != len(data) {
+		return nil, fmt.Errorf("serialized image data has %d frames in metadata but have data for: %d", len(m.Frames), len(data))
+	}
+	ans := ImageData{
+		Width: m.Width, Height: m.Height, Format_uppercase: m.Format_uppercase,
+	}
+	for i, f := range m.Frames {
+		if ff, err := ImageFrameFromSerialized(f, data[i]); err != nil {
+			return nil, err
+		} else {
+			ans.Frames = append(ans.Frames, ff)
+		}
+	}
+	return &ans, nil
+}
+
 func (self *ImageFrame) Resize(x_frac, y_frac float64) *ImageFrame {
 	b := self.Img.Bounds()
 	left, top, width, height := b.Min.X, b.Min.Y, b.Dx(), b.Dy()
@@ -194,29 +286,7 @@ func MakeTempDir(template string) (ans string, err error) {
 	return os.MkdirTemp("", template)
 }
 
-func check_resize(frame *ImageFrame, filename string) error {
-	// ImageMagick sometimes generates RGBA images smaller than the specified
-	// size. See https://github.com/kovidgoyal/kitty/issues/276 for examples
-	s, err := os.Stat(filename)
-	if err != nil {
-		return err
-	}
-	sz := int(s.Size())
-	bytes_per_pixel := 4
-	if frame.Is_opaque {
-		bytes_per_pixel = 3
-	}
-	expected_size := bytes_per_pixel * frame.Width * frame.Height
-	if sz < expected_size {
-		missing := expected_size - sz
-		if missing%(bytes_per_pixel*frame.Width) != 0 {
-			return fmt.Errorf("ImageMagick failed to resize correctly. It generated %d < %d of data (w=%d h=%d bpp=%d)", sz, expected_size, frame.Width, frame.Height, bytes_per_pixel)
-		}
-		frame.Height -= missing / (bytes_per_pixel * frame.Width)
-	}
-	return nil
-}
-
+// Native {{{
 func (frame *ImageFrame) set_delay(min_gap, delay int) {
 	frame.Delay_ms = int32(max(min_gap, delay) * 10)
 	if frame.Delay_ms == 0 {
@@ -266,10 +336,36 @@ func OpenNativeImageFromReader(f io.ReadSeeker) (ans *ImageData, err error) {
 	return
 }
 
+// }}}
+
+// ImageMagick {{{
 var MagickExe = sync.OnceValue(func() string {
 	return utils.FindExe("magick")
 })
 
+func check_resize(frame *ImageFrame, filename string) error {
+	// ImageMagick sometimes generates RGBA images smaller than the specified
+	// size. See https://github.com/kovidgoyal/kitty/issues/276 for examples
+	s, err := os.Stat(filename)
+	if err != nil {
+		return err
+	}
+	sz := int(s.Size())
+	bytes_per_pixel := 4
+	if frame.Is_opaque {
+		bytes_per_pixel = 3
+	}
+	expected_size := bytes_per_pixel * frame.Width * frame.Height
+	if sz < expected_size {
+		missing := expected_size - sz
+		if missing%(bytes_per_pixel*frame.Width) != 0 {
+			return fmt.Errorf("ImageMagick failed to resize correctly. It generated %d < %d of data (w=%d h=%d bpp=%d)", sz, expected_size, frame.Width, frame.Height, bytes_per_pixel)
+		}
+		frame.Height -= missing / (bytes_per_pixel * frame.Width)
+	}
+	return nil
+}
+
 func RunMagick(path string, cmd []string) ([]byte, error) {
 	if MagickExe() != "magick" {
 		cmd = append([]string{MagickExe()}, cmd...)
@@ -428,7 +524,7 @@ func IdentifyWithMagick(path string) (ans []IdentifyRecord, err error) {
 }
 
 type RenderOptions struct {
-	RemoveAlpha          *NRGBColor
+	RemoveAlpha          *imaging.NRGBColor
 	Flip, Flop           bool
 	ResizeTo             image.Point
 	OnlyFirstFrame       bool
@@ -599,7 +695,7 @@ func OpenImageFromPathWithMagick(path string) (ans *ImageData, err error) {
 		}
 		dest_rect := image.Rect(0, 0, frame.Width, frame.Height)
 		if frame.Is_opaque {
-			frame.Img = &NRGB{Pix: data, Stride: frame.Width * 3, Rect: dest_rect}
+			frame.Img = &imaging.NRGB{Pix: data, Stride: frame.Width * 3, Rect: dest_rect}
 		} else {
 			frame.Img = &image.NRGBA{Pix: data, Stride: frame.Width * 4, Rect: dest_rect}
 		}
@@ -610,6 +706,8 @@ func OpenImageFromPathWithMagick(path string) (ans *ImageData, err error) {
 	return ans, nil
 }
 
+// }}}
+
 func OpenImageFromPath(path string) (ans *ImageData, err error) {
 	mt := utils.GuessMimeType(path)
 	if DecodableImageTypes[mt] {
@@ -620,7 +718,7 @@ func OpenImageFromPath(path string) (ans *ImageData, err error) {
 		defer f.Close()
 		ans, err = OpenNativeImageFromReader(f)
 		if err != nil {
-			return nil, fmt.Errorf("Failed to load image at %#v with error: %w", path, err)
+			return OpenImageFromPathWithMagick(path)
 		}
 	} else {
 		return OpenImageFromPathWithMagick(path)
