@@ -6,7 +6,7 @@ import (
 	"bytes"
 	"fmt"
 
-	"kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 var _ = fmt.Print
